@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,9 @@
 """Wrapper for user-visible error exceptions to raise in the CLI."""
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.util import exceptions as api_lib_exceptions
 from googlecloudsdk.core import exceptions as core_exceptions
 
@@ -32,7 +34,11 @@ class ListServicesPermissionDeniedException(Error):
   pass
 
 
-class PeerServicePermissionDeniedException(Error):
+class CreateConnectionsPermissionDeniedException(Error):
+  pass
+
+
+class ListConnectionsPermissionDeniedException(Error):
   pass
 
 
