@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,7 +20,6 @@ from __future__ import division
 from __future__ import unicode_literals
 
 import functools
-import enum  # pylint: disable=unused-import, for pytype
 
 from googlecloudsdk.api_lib.compute import filter_rewrite
 from googlecloudsdk.api_lib.compute.regions import service as regions_service
@@ -237,7 +236,6 @@ class ResourceArgScopes(object):
     self.scopes = {}
 
   def AddScope(self, scope, collection):
-    # type: (enum.Enum, str) -> None
     self.scopes[scope] = ResourceArgScope(scope, self.flag_prefix, collection)
 
   def SpecifiedByArgs(self, args):
@@ -510,7 +508,7 @@ class ResourceResolver(object):
         api_resource_registry)
 
     # Now unpack each element.
-    refs = [ref[0] for ref in refs]  # type: list[resources.Resource]
+    refs = [ref[0] for ref in refs]
 
     # Make sure correct collection was given for each resource, for example
     # URLs have implicit collections.
@@ -795,6 +793,17 @@ def AddStorageLocationFlag(parser, resource):
       """.format(resource))
 
 
+def AddGuestFlushFlag(parser, resource):
+  parser.add_argument(
+      '--guest-flush',
+      action='store_true',
+      default=False,
+      help=('Create an application-consistent {} by informing the OS '
+            'to prepare for the snapshot process. Currently only supported '
+            'on Windows instances using the Volume Shadow Copy Service '
+            '(VSS).'.format(resource)))
+
+
 def AddShieldedInstanceInitialStateKeyArg(parser):
   """Add the initial state for shielded instance arg."""
   parser.add_argument(
@@ -858,3 +867,40 @@ def RewriteFilter(args, message=None, frontend_fields=None):
           args.filter, defaults=defaults)
   log.info('client_filter=%r server_filter=%r', client_filter, server_filter)
   return client_filter, server_filter
+
+
+def AddSourceDiskCsekKeyArg(parser):
+  spec = {
+      'disk': str,
+      'csek-key-file': str
+  }
+  parser.add_argument(
+      '--source-disk-csek-key',
+      type=arg_parsers.ArgDict(spec=spec),
+      action='append',
+      metavar='PROPERTY=VALUE',
+      help="""
+              Customer-supplied encryption key of the disk attached to the
+              source instance. Required if the source disk is protected by
+              a customer-supplied encryption key. This flag may be repeated to
+              specify multiple attached disks.
+
+              *disk*::: URL of the disk attached to the source instance.
+              This can be a full or   valid partial URL
+
+              *csek-key-file*::: path to customer-supplied encryption key.
+            """
+  )
+
+
+def AddEraseVssSignature(parser, resource):
+  parser.add_argument(
+      '--erase-windows-vss-signature',
+      action='store_true',
+      default=False,
+      help="""
+              Specifies whether the disk restored from {resource} should
+              erase Windows specific VSS signature. See
+                *https://cloud.google.com/sdk/gcloud/reference/compute/disks/snapshot#--guest-flush
+           """.format(resource=resource)
+  )
