@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -40,20 +40,22 @@ FORWARDING_RULES_OVERVIEW = """\
         target pools, target instances and target VPN gateways and backend
         services.
 
-        Forwarding rules can be either external or internal, specified with the
-        ``--load-balancing-scheme=[EXTERNAL|INTERNAL]'' flag. External
-        forwarding rules are accessible from the internet, while internal
-        forwarding rules are only accessible from within their VPC networks. You
-        can specify a reserved static external or internal IP address with the
-        ``--address=ADDRESS'' flag for the forwarding rule. Otherwise if the
-        flag is unspecified, an external forwarding rule will be automatically
-        assigned an ephemeral external IP address (global IP addresses for
-        global forwarding rules and regional IP addresses for regional
-        forwarding rules); an internal forwarding rule will be automatically
-        assigned an ephemeral internal IP address from the subnet specified with
-        the ``--subnet'' flag.
+        Forwarding rules can be either external, internal or internal self
+        managed, specified with the
+        ``--load-balancing-scheme=[EXTERNAL|INTERNAL|INTERNAL_SELF_MANAGED]''
+        flag. External forwarding rules are accessible from the internet, while
+        internal forwarding rules are only accessible from within their VPC
+        networks. You can specify a reserved static external or internal IP
+        address with the ``--address=ADDRESS'' flag for the forwarding rule.
+        Otherwise if the flag is unspecified, an external forwarding rule will
+        be automatically assigned an ephemeral external IP address (global IP
+        addresses for global forwarding rules and regional IP addresses for
+        regional forwarding rules); an internal forwarding rule will be
+        automatically assigned an ephemeral internal IP address from the subnet
+        specified with the ``--subnet'' flag. An IP Address must be provided for
+        an internal self managed forwarding rule.
 
-        There are different types of load balancer working at different layers
+        There are different types of load balancers working at different layers
         of the OSI networking model
         (http://en.wikipedia.org/wiki/Network_layer). Layer 3 load balancer
         targets include target pools, target SSL proxies, target TCP proxies and
@@ -174,33 +176,29 @@ BACKEND_SERVICE_ARG = compute_flags.ResourceArgument(
     region_explanation=('If not specified, it will be set to the'
                         ' region of the forwarding rule.'))
 
-NETWORK_ARG_ALPHA = compute_flags.ResourceArgument(
-    name='--network',
-    required=False,
-    resource_name='networks',
-    global_collection='compute.networks',
-    short_help='Network that this forwarding rule applies to.',
-    detailed_help="""\
-        (Only for --load-balancing-scheme=INTERNAL,
-        --load-balancing-scheme=INTERNAL_MANAGED, or
-        --load-balancing-scheme=INTERNAL_SELF_MANAGED) Network that this
-        forwarding rule applies to. If this field is not specified, the default
-        network will be used. In the absence of the default network, this field
-        must be specified.
-        """)
 
-NETWORK_ARG = compute_flags.ResourceArgument(
-    name='--network',
-    required=False,
-    resource_name='networks',
-    global_collection='compute.networks',
-    short_help='Network that this forwarding rule applies to.',
-    detailed_help="""\
-        (Only for --load-balancing-scheme=INTERNAL) Network that this
-        forwarding rule applies to. If this field is not specified, the default
-        network will be used. In the absence of the default network, this field
-        must be specified.
-        """)
+def NetworkArg(include_l7_internal_load_balancing):
+  """Returns the network parameter."""
+
+  load_balancing_scheme = ('--load-balancing-scheme=INTERNAL or ' +
+                           '--load-balancing-scheme=INTERNAL_SELF_MANAGED')
+
+  if include_l7_internal_load_balancing:
+    load_balancing_scheme += ' or --load-balancing-scheme=INTERNAL_MANAGED'
+
+  return compute_flags.ResourceArgument(
+      name='--network',
+      required=False,
+      resource_name='networks',
+      global_collection='compute.networks',
+      short_help='Network that this forwarding rule applies to.',
+      detailed_help="""
+          (Only for %s) Network that this
+          forwarding rule applies to. If this field is not specified, the default
+          network will be used. In the absence of the default network, this field
+          must be specified.
+          """ % load_balancing_scheme)
+
 
 SUBNET_ARG = compute_flags.ResourceArgument(
     name='--subnet',
@@ -219,7 +217,7 @@ SUBNET_ARG = compute_flags.ResourceArgument(
                         ' region of the forwarding rule.'))
 
 
-def TargetHttpProxyArg(include_alpha=False):
+def TargetHttpProxyArg(include_l7_internal_load_balancing=False):
   """Return a resource argument for parsing a target http proxy."""
 
   target_http_proxy_arg = compute_flags.ResourceArgument(
@@ -228,16 +226,16 @@ def TargetHttpProxyArg(include_alpha=False):
       resource_name='http proxy',
       global_collection='compute.targetHttpProxies',
       regional_collection='compute.regionTargetHttpProxies'
-      if include_alpha else None,
+      if include_l7_internal_load_balancing else None,
       short_help='Target HTTP proxy that will receive the traffic.',
       detailed_help=('Target HTTP proxy that will receive the traffic. '
                      'Acceptable values for --ports flag are: 80, 8080.'),
       region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
-      if include_alpha else None)
+      if include_l7_internal_load_balancing else None)
   return target_http_proxy_arg
 
 
-def TargetHttpsProxyArg(include_alpha=False):
+def TargetHttpsProxyArg(include_l7_internal_load_balancing=False):
   """Return a resource argument for parsing a target https proxy."""
 
   target_https_proxy_arg = compute_flags.ResourceArgument(
@@ -246,12 +244,12 @@ def TargetHttpsProxyArg(include_alpha=False):
       resource_name='https proxy',
       global_collection='compute.targetHttpsProxies',
       regional_collection='compute.regionTargetHttpsProxies'
-      if include_alpha else None,
+      if include_l7_internal_load_balancing else None,
       short_help='Target HTTPS proxy that will receive the traffic.',
       detailed_help=('Target HTTPS proxy that will receive the traffic. '
                      'Acceptable values for --ports flag are: 443.'),
       region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
-      if include_alpha else None)
+      if include_l7_internal_load_balancing else None)
   return target_https_proxy_arg
 
 TARGET_INSTANCE_ARG = compute_flags.ResourceArgument(
@@ -316,9 +314,14 @@ TARGET_VPN_GATEWAY_ARG = compute_flags.ResourceArgument(
                         ' region of the forwarding rule.'))
 
 
-def AddressArgHelp(include_alpha):
+def AddressArgHelp(include_l7_internal_load_balancing):
   """Build the help text for the address argument."""
 
+  lb_schemes = '(EXTERNAL, INTERNAL, INTERNAL_MANAGED'
+  if include_l7_internal_load_balancing:
+    lb_schemes += ', INTERNAL_MANAGED'
+  lb_schemes += ')'
+
   detailed_help = """\
     IP address that the forwarding rule will serve. All
     traffic sent to this IP address is directed to the target
@@ -342,7 +345,10 @@ def AddressArgHelp(include_alpha):
     for the forwarding rule. If this flag is omitted, an ephemeral internal IP
     address will be automatically allocated from the IP range of the subnet or
     network configured for this forwarding rule.
-    %s
+
+    When the --load-balancing-scheme is INTERNAL_SELF_MANAGED, this must
+    be a URL reference to an existing Address resource.
+
     Note: An IP address must be specified if the traffic is being forwarded to
     a VPN.
 
@@ -354,46 +360,39 @@ def AddressArgHelp(include_alpha):
     - regions/us-central1/addresses/address-1
     - global/addresses/address-1
     - address-1
-  """ % ('EXTERNAL, INTERNAL, INTERNAL_MANAGED or INTERNAL_SELF_MANAGED'
-         if include_alpha else 'EXTERNAL or INTERNAL', """
-    When the --load-balancing-scheme is INTERNAL_SELF_MANAGED, this must
-    be a URL reference to an existing Address resource.
-         """ if include_alpha else '')
+  """ % (
+      lb_schemes)
 
   return textwrap.dedent(detailed_help)
 
 
-ADDRESS_ARG_ALPHA = compute_flags.ResourceArgument(
-    name='--address',
-    required=False,
-    resource_name='address',
-    completer=addresses_flags.AddressesCompleter,
-    regional_collection='compute.addresses',
-    global_collection='compute.globalAddresses',
-    region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION,
-    short_help='IP address that the forwarding rule will serve.',
-    detailed_help=AddressArgHelp(include_alpha=True))
-
-ADDRESS_ARG = compute_flags.ResourceArgument(
-    name='--address',
-    required=False,
-    resource_name='address',
-    completer=addresses_flags.AddressesCompleter,
-    regional_collection='compute.addresses',
-    global_collection='compute.globalAddresses',
-    region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION,
-    short_help='IP address that the forwarding rule will serve.',
-    detailed_help=AddressArgHelp(include_alpha=False))
+def AddressArg(include_l7_internal_load_balancing):
+  return compute_flags.ResourceArgument(
+      name='--address',
+      required=False,
+      resource_name='address',
+      completer=addresses_flags.AddressesCompleter,
+      regional_collection='compute.addresses',
+      global_collection='compute.globalAddresses',
+      region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION,
+      short_help='IP address that the forwarding rule will serve.',
+      detailed_help=AddressArgHelp(
+          include_l7_internal_load_balancing=include_l7_internal_load_balancing)
+  )
 
 
-def AddUpdateArgs(parser, include_beta=False, include_alpha=False):
+def AddUpdateArgs(parser,
+                  include_l7_internal_load_balancing=False):
   """Adds common flags for mutating forwarding rule targets."""
-  del include_beta
   target = parser.add_mutually_exclusive_group(required=True)
 
-  TargetHttpProxyArg(include_alpha=include_alpha).AddArgument(
+  TargetHttpProxyArg(
+      include_l7_internal_load_balancing=include_l7_internal_load_balancing
+  ).AddArgument(
       parser, mutex_group=target)
-  TargetHttpsProxyArg(include_alpha=include_alpha).AddArgument(
+  TargetHttpsProxyArg(
+      include_l7_internal_load_balancing=include_l7_internal_load_balancing
+  ).AddArgument(
       parser, mutex_group=target)
   TARGET_INSTANCE_ARG.AddArgument(parser, mutex_group=target)
   TARGET_POOL_ARG.AddArgument(parser, mutex_group=target)
@@ -402,16 +401,17 @@ def AddUpdateArgs(parser, include_beta=False, include_alpha=False):
   TARGET_VPN_GATEWAY_ARG.AddArgument(parser, mutex_group=target)
 
   BACKEND_SERVICE_ARG.AddArgument(parser, mutex_group=target)
-  if include_alpha:
-    NETWORK_ARG_ALPHA.AddArgument(parser)
-  else:
-    NETWORK_ARG.AddArgument(parser)
+  NetworkArg(
+      include_l7_internal_load_balancing=include_l7_internal_load_balancing
+  ).AddArgument(parser)
   SUBNET_ARG.AddArgument(parser)
 
-  AddLoadBalancingScheme(parser, include_alpha)
+  AddLoadBalancingScheme(
+      parser,
+      include_l7_ilb=include_l7_internal_load_balancing)
 
 
-def AddLoadBalancingScheme(parser, include_alpha=False):
+def AddLoadBalancingScheme(parser, include_l7_ilb=False):
   """Adds the load-balancing-scheme flag."""
   load_balancing_choices = {
       'EXTERNAL':
@@ -421,12 +421,13 @@ def AddLoadBalancingScheme(parser, include_alpha=False):
           '--target-instance.',
       'INTERNAL': 'Internal load balancing or forwarding, used with '
                   '--backend-service.',
+      'INTERNAL_SELF_MANAGED':
+          'Traffic director load balancing or forwarding, used with '
+          '--target-http-proxy, --target-https-proxy.'
   }
-  if include_alpha:
+
+  if include_l7_ilb:
     load_balancing_choices.update({
-        'INTERNAL_SELF_MANAGED':
-            'Traffic director load balancing or forwarding, used with '
-            '--target-http-proxy, --target-https-proxy.',
         'INTERNAL_MANAGED': 'Internal HTTP(S) Load Balancing, used with '
                             '--target-http-proxy, --target-https-proxy.'
     })
@@ -485,14 +486,15 @@ def AddIpVersionGroup(parser):
       """)
 
 
-def AddAddressesAndIPVersions(parser, required=True, include_alpha=False):
+def AddAddressesAndIPVersions(parser, required,
+                              include_l7_internal_load_balancing):
   """Adds Addresses and IP versions flag."""
+
+  address_arg = AddressArg(
+      include_l7_internal_load_balancing=include_l7_internal_load_balancing)
   group = parser.add_mutually_exclusive_group(required=required)
   AddIpVersionGroup(group)
-  if include_alpha:
-    ADDRESS_ARG_ALPHA.AddArgument(parser, mutex_group=group)
-  else:
-    ADDRESS_ARG.AddArgument(parser, mutex_group=group)
+  address_arg.AddArgument(parser, mutex_group=group)
 
 
 def AddDescription(parser):
