@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,6 +28,7 @@ import select
 import socket
 import sys
 import threading
+import time
 
 from googlecloudsdk.api_lib.compute import iap_tunnel_websocket
 from googlecloudsdk.api_lib.compute import iap_tunnel_websocket_utils as utils
@@ -99,7 +100,8 @@ class SshTunnelArgs(object):
     self.pass_through_args = []
 
   @staticmethod
-  def FromArgs(args, track, instance_ref, interface):
+  def FromArgs(args, track, instance_ref, internal_interface,
+               external_interface):
     """Construct an SshTunnelArgs from command line args and values.
 
     Args:
@@ -107,20 +109,39 @@ class SshTunnelArgs(object):
         AddSshTunnelArgs called.
       track: ReleaseTrack, The currently running release track.
       instance_ref: The target instance reference object.
-      interface: The target interface resource object.
+      internal_interface: The internal interface of target resource object.
+      external_interface: The external interface of target resource object, if
+        available, otherwise None.
     Returns:
       SshTunnelArgs or None if IAP Tunnel is disabled.
     """
-    if not getattr(args, 'tunnel_through_iap', False):
+    # If tunneling through IAP is not available, then abort.
+    if not hasattr(args, 'tunnel_through_iap'):
       return None
 
+    # If set to connect directly to private IP address, then abort.
+    if getattr(args, 'internal_ip', False):
+      return None
+
+    if args.IsSpecified('tunnel_through_iap'):
+      # If IAP tunneling is explicitly disabled, then abort.
+      if not args.tunnel_through_iap:
+        return None
+    else:
+      # If no external interface is available, then default to using IAP
+      # tunneling and continue with code below.  Otherwise, abort.
+      if external_interface:
+        return None
+      log.status.Print('External IP address was not found; defaulting to using '
+                       'IAP tunneling.')
+
     res = SshTunnelArgs()
 
     res.track = track.prefix
     res.project = instance_ref.project
     res.zone = instance_ref.zone
     res.instance = instance_ref.instance
-    res.interface = interface.name
+    res.interface = internal_interface.name
 
     # The tunnel_through_iap attribute existed, so these must too.
     if args.IsSpecified('iap_tunnel_url_override'):
@@ -162,8 +183,17 @@ def DetermineLocalPort(port_arg=0):
 
 
 def _CloseLocalConnectionCallback(local_conn):
+  """Callback function to close the local connection, if any."""
   # For test WebSocket connections, there is not a local socket connection.
   if local_conn:
+    try:
+      # Calling shutdown() first is needed to promptly notify the process on
+      # the other side of the connection that it is closing. This allows that
+      # other process, whether over TCP or stdin, to promptly terminate rather
+      # that waiting for the next time that the process tries to send data.
+      local_conn.shutdown(socket.SHUT_RDWR)
+    except EnvironmentError:
+      pass
     try:
       local_conn.close()
     except EnvironmentError:
@@ -221,7 +251,7 @@ class _StdinSocket(object):
 
   def __init__(self):
     # This is only used in Unix.
-    self._stdin_eof = False
+    self._stdin_closed = False
 
   def send(self, data):  # pylint: disable=invalid-name
     files.WriteStreamBytes(sys.stdout, data)
@@ -257,9 +287,14 @@ class _StdinSocket(object):
     # Closing stdin doesn't help, because it doesn't unblock read() calls.
     # Also it causes problems, such as segfaulting in python2 and blocking in
     # python3.
-    pass
+    self.shutdown(socket.SHUT_RD)
+
+  def shutdown(self, how):  # pylint: disable=invalid-name
+    # Shutting down read only (SHUT_RD) on Unix only (no change/effect on
+    # Windows)
+    if how in (socket.SHUT_RDWR, socket.SHUT_RD):
+      self._stdin_closed = True
 
-  # pytype: disable=module-attr
   def _RecvWindows(self, bufsize):
     """Reads data from std in Windows.
 
@@ -274,13 +309,12 @@ class _StdinSocket(object):
     # STD_INPUT_HANDLE is -10
     h = ctypes.windll.kernel32.GetStdHandle(-10)
     buf = ctypes.create_string_buffer(bufsize)
-    number_of_bytes_read = wintypes.DWORD()  # pytype: disable=not-callable
+    number_of_bytes_read = wintypes.DWORD()
     ok = ctypes.windll.kernel32.ReadFile(
         h, buf, bufsize, ctypes.byref(number_of_bytes_read), None)
     if not ok:
       raise socket.error(errno.EIO, 'stdin ReadFile failed')
     return buf.raw[:number_of_bytes_read.value]
-  # pytype: enable=module-attr
 
   class _EOFError(Exception):
     pass
@@ -298,39 +332,14 @@ class _StdinSocket(object):
     # is to make stdin non-blocking. To ensure at least 1 byte is received, we
     # read the first byte blocking.
     b = b''
-    if self._stdin_eof:
-      return b
     try:
-      b += self._ReadUnixBlocking(1)
-      if bufsize > 1:
-        b += self._ReadUnixNonBlocking(bufsize-1)
+      while not self._stdin_closed:
+        b = self._ReadUnixNonBlocking(bufsize)
+        if b:
+          break
+        time.sleep(0.001)
     except _StdinSocket._EOFError:
-      # We need to remember if an EOF is received because additional data can
-      # be received after EOF, and our 2 reads and concatenation could otherwise
-      # hide from the caller that an EOF was received.
-      self._stdin_eof = True
-    return b
-
-  def _ReadUnixBlocking(self, bufsize):
-    """Reads from stdin on Unix in a blocking manner.
-
-    Args:
-      bufsize: The maximum number of bytes to receive. Must be positive.
-    Returns:
-      The bytes read.
-    Raises:
-      _StdinSocket._EOFError: to indicate EOF.
-    """
-    # In python 3, we need to read stdin in a binary way, not a text way to
-    # read bytes instead of str. In python 2, binary mode vs text mode only
-    # matters on Windows.
-    if six.PY2:
-      b = sys.stdin.read(bufsize)
-    else:
-      b = sys.stdin.buffer.read(bufsize)
-    if not b:
-      # In python 2 and 3, EOF is indicated by returning b''.
-      raise _StdinSocket._EOFError
+      self._stdin_closed = True
     return b
 
   def _ReadUnixNonBlocking(self, bufsize):
@@ -343,6 +352,9 @@ class _StdinSocket(object):
     Raises:
       _StdinSocket._EOFError: to indicate EOF.
     """
+    # In python 3, we need to read stdin in a binary way, not a text way to
+    # read bytes instead of str. In python 2, binary mode vs text mode only
+    # matters on Windows.
     import fcntl  # pylint: disable=g-import-not-at-top
     old_flags = fcntl.fcntl(sys.stdin, fcntl.F_GETFL)
     try:
