@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,6 +32,19 @@ IP_ADDRESSES_ARG = compute_flags.ResourceArgument(
     plural=True,
     required=False)
 
+DRAIN_NAT_IP_ADDRESSES_ARG = compute_flags.ResourceArgument(
+    name='--drain-nat-ips',
+    detailed_help=textwrap.dedent("""\
+       External IP Addresses to be drained
+
+       These IPs must be valid external IPs that have been used as NAT IPs
+       """),
+    resource_name='address',
+    regional_collection='compute.addresses',
+    region_hidden=True,
+    plural=True,
+    required=False)
+
 
 class IpAllocationOption(enum.Enum):
   AUTO = 0
@@ -63,14 +76,17 @@ def AddNatNameArg(parser, operation_type='operate on', plural=False):
   parser.add_argument('name', **params)
 
 
-def AddCommonNatArgs(parser, for_create=False, with_logging=False):
+def AddCommonNatArgs(parser,
+                     for_create=False,
+                     with_drain_ips=False):
   """Adds common arguments for creating and updating NATs."""
   _AddIpAllocationArgs(parser, for_create)
   _AddSubnetworkArgs(parser, for_create)
   _AddTimeoutsArgs(parser, for_create)
   _AddMinPortsPerVmArg(parser, for_create)
-  if with_logging:
-    _AddLoggingArgs(parser, for_create)
+  _AddLoggingArgs(parser)
+  if with_drain_ips and not for_create:
+    _AddDrainNatIpsArgument(parser)
 
 
 def _AddIpAllocationArgs(parser, for_create=False):
@@ -165,7 +181,7 @@ def _AddMinPortsPerVmArg(parser, for_create=False):
       'Clear minimum ports to be allocated to a VM')
 
 
-def _AddLoggingArgs(parser, for_create=False):
+def _AddLoggingArgs(parser):
   """Adds arguments to configure NAT logging."""
   enable_logging_help_text = textwrap.dedent("""\
     Enable logging for the NAT. Logs will be exported to Stackdriver. NAT
@@ -174,25 +190,17 @@ def _AddLoggingArgs(parser, for_create=False):
     $ {parent_command} update MY-NAT --no-enable-logging --router ROUTER
       --region REGION""")
   log_filter_help_text = textwrap.dedent("""\
-    Specify a filter for logs exported to stackdriver. This can be one of the
-    following:
-    ERRORS_ONLY - export logs for connection failures only.
-    TRANSLATIONS_ONLY - export logs for successful connections only.
-    Without a filter, logs (if enabled) will be exported for all connections
-    handled by this NAT.
+    Filter for logs exported to stackdriver.
+
+    The default is ALL.
 
     If logging is not enabled, filter settings will be persisted but will have
     no effect.
-    Use --[no-]enable-logging to enable and disable logging.
 
-    To clear the filter, use
-    $ {parent_command} update MY-NAT --clear-log-filter --router ROUTER
-      --region REGION""")
-  clear_log_filter_help_text = textwrap.dedent("""\
-    Clear the filter for logs exported to stackdriver.
-    Without a filter, logs (if enabled) will be exported for all connections
-    handled by this NAT.""")
+    Use --[no-]enable-logging to enable and disable logging.
+""")
   filter_choices = {
+      'ALL': 'Export logs for all connections handled by this NAT.',
       'ERRORS_ONLY': 'Export logs for connection failures only.',
       'TRANSLATIONS_ONLY': 'Export logs for successful connections only.',
   }
@@ -201,9 +209,18 @@ def _AddLoggingArgs(parser, for_create=False):
       action='store_true',
       default=None,
       help=enable_logging_help_text)
-  _AddClearableArgument(parser, for_create, 'log-filter', None,
-                        log_filter_help_text, clear_log_filter_help_text,
-                        filter_choices)
+  parser.add_argument(
+      '--log-filter', help=log_filter_help_text, choices=filter_choices)
+
+
+def _AddDrainNatIpsArgument(parser):
+  drain_ips_group = parser.add_mutually_exclusive_group(required=False)
+  DRAIN_NAT_IP_ADDRESSES_ARG.AddArgument(parser, mutex_group=drain_ips_group)
+  drain_ips_group.add_argument(
+      '--clear-drain-nat-ips',
+      action='store_true',
+      default=False,
+      help='Clear the drained NAT IPs')
 
 
 def _AddClearableArgument(parser,
