@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -43,7 +43,7 @@ def FindNatOrRaise(router, nat_name):
   raise NatNotFoundError(nat_name)
 
 
-def CreateNatMessage(args, compute_holder, with_logging=False):
+def CreateNatMessage(args, compute_holder):
   """Creates a NAT message from the specified arguments."""
   params = {'name': args.name}
 
@@ -59,20 +59,22 @@ def CreateNatMessage(args, compute_holder, with_logging=False):
   params['tcpEstablishedIdleTimeoutSec'] = args.tcp_established_idle_timeout
   params['tcpTransitoryIdleTimeoutSec'] = args.tcp_transitory_idle_timeout
   params['minPortsPerVm'] = args.min_ports_per_vm
-  if with_logging:
-    if args.enable_logging is not None or args.log_filter is not None:
-      log_config = compute_holder.client.messages.RouterNatLogConfig()
+  if args.enable_logging is not None or args.log_filter is not None:
+    log_config = compute_holder.client.messages.RouterNatLogConfig()
 
-      log_config.enable = args.enable_logging
-      if args.log_filter is not None:
-        log_config.filter = _TranslateLogFilter(args.log_filter, compute_holder)
+    log_config.enable = args.enable_logging
+    if args.log_filter is not None:
+      log_config.filter = _TranslateLogFilter(args.log_filter, compute_holder)
 
-      params['logConfig'] = log_config
+    params['logConfig'] = log_config
 
   return compute_holder.client.messages.RouterNat(**params)
 
 
-def UpdateNatMessage(nat, args, compute_holder, with_logging=False):
+def UpdateNatMessage(nat,
+                     args,
+                     compute_holder,
+                     with_drain_ips=False):
   """Updates a NAT message with the specified arguments."""
   if (args.subnet_option in [
       nat_flags.SubnetOption.ALL_RANGES, nat_flags.SubnetOption.PRIMARY_RANGES
@@ -81,6 +83,20 @@ def UpdateNatMessage(nat, args, compute_holder, with_logging=False):
     nat.sourceSubnetworkIpRangesToNat = ranges_to_nat
     nat.subnetworks = subnetworks
 
+  if with_drain_ips:
+    if args.drain_nat_ips:
+      drain_nat_ips = nat_flags.DRAIN_NAT_IP_ADDRESSES_ARG.ResolveAsResource(
+          args, compute_holder.resources)
+      nat.drainNatIps = [str(ip) for ip in drain_nat_ips]
+
+      # Remove a IP from nat_ips if it is going to be drained.
+      if not args.nat_external_ip_pool:
+        nat.natIps = [ip for ip in nat.natIps
+                      if not _ContainIp(drain_nat_ips, ip)]
+
+    if args.clear_drain_nat_ips:
+      nat.drainNatIps = []
+
   if (args.ip_allocation_option == nat_flags.IpAllocationOption.AUTO or
       args.nat_external_ip_pool):
     option, nat_ips = _ParseNatIpFields(args, compute_holder)
@@ -112,17 +128,13 @@ def UpdateNatMessage(nat, args, compute_holder, with_logging=False):
   elif args.clear_min_ports_per_vm:
     nat.minPortsPerVm = None
 
-  if with_logging:
-    if args.enable_logging is not None or args.log_filter is not None:
-      nat.logConfig = (
-          nat.logConfig or compute_holder.client.messages.RouterNatLogConfig())
-    if args.enable_logging is not None:
-      nat.logConfig.enable = args.enable_logging
-    if args.log_filter is not None:
-      nat.logConfig.filter = _TranslateLogFilter(args.log_filter,
-                                                 compute_holder)
-    elif args.clear_log_filter:
-      nat.logConfig.filter = None
+  if args.enable_logging is not None or args.log_filter is not None:
+    nat.logConfig = (
+        nat.logConfig or compute_holder.client.messages.RouterNatLogConfig())
+  if args.enable_logging is not None:
+    nat.logConfig.enable = args.enable_logging
+  if args.log_filter is not None:
+    nat.logConfig.filter = _TranslateLogFilter(args.log_filter, compute_holder)
 
   return nat
 
@@ -219,6 +231,9 @@ def _ParseNatIpFields(args, compute_holder):
 
 def _TranslateLogFilter(filter_str, compute_holder):
   """Translates the specified log filter to the enum value."""
+  if filter_str == 'ALL':
+    return (compute_holder.client.messages.RouterNatLogConfig
+            .FilterValueValuesEnum.ALL)
   if filter_str == 'TRANSLATIONS_ONLY':
     return (compute_holder.client.messages.RouterNatLogConfig
             .FilterValueValuesEnum.TRANSLATIONS_ONLY)
@@ -227,5 +242,13 @@ def _TranslateLogFilter(filter_str, compute_holder):
             .FilterValueValuesEnum.ERRORS_ONLY)
 
   raise calliope_exceptions.InvalidArgumentException(
-      '--log-filter', ('--log-filter must be TRANSLATIONS_ONLY '
+      '--log-filter', ('--log-filter must be ALL, TRANSLATIONS_ONLY '
                        'or ERRORS_ONLY'))
+
+
+def _ContainIp(ip_list, target_ip):
+  """Returns true if target ip is in the list."""
+  for ip in ip_list:
+    if ip.RelativeName() in target_ip:
+      return True
+  return False
