@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ from argcomplete.completers import FilesCompleter
 from googlecloudsdk.calliope import actions
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import flags
+from googlecloudsdk.command_lib.compute import iap_tunnel
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute import ssh_utils
 from googlecloudsdk.command_lib.compute.instances import flags as instance_flags
@@ -135,34 +136,25 @@ class BaseScpHelper(ssh_utils.BaseSSHCLIHelper):
       options = self.GetConfig(ssh_utils.HostKeyAlias(instance),
                                args.strict_host_key_checking)
 
-    tunnel_helper = None
-    cmd_port = port
-    if hasattr(args, 'tunnel_through_iap') and args.tunnel_through_iap:
-      tunnel_helper = ssh_utils.CreateIapTunnelHelper(args, instance_ref,
-                                                      instance, port=port)
-      tunnel_helper.StartListener()
-      cmd_port = str(tunnel_helper.GetLocalPort())
-      if dst.remote:
-        dst.remote.host = 'localhost'
-      else:
-        for src in srcs:
-          src.remote.host = 'localhost'
+    iap_tunnel_args = iap_tunnel.SshTunnelArgs.FromArgs(
+        args, release_track, instance_ref,
+        ssh_utils.GetInternalInterface(instance),
+        ssh_utils.GetExternalInterface(instance, no_raise=True))
+
+    if iap_tunnel_args:
+      remote.host = ssh_utils.HostKeyAlias(instance)
+    elif ip_type is ip.IpTypeEnum.INTERNAL:
+      remote.host = ssh_utils.GetInternalIPAddress(instance)
     else:
-      # Now replace the instance name with the actual IP/hostname
-      if ip_type is ip.IpTypeEnum.INTERNAL:
-        remote.host = ssh_utils.GetInternalIPAddress(instance)
-      else:
-        remote.host = ssh_utils.GetExternalIPAddress(instance)
+      remote.host = ssh_utils.GetExternalIPAddress(instance)
 
     cmd = ssh.SCPCommand(
         srcs, dst, identity_file=identity_file, options=options,
-        recursive=recursive, compress=compress, port=cmd_port,
-        extra_flags=extra_flags)
+        recursive=recursive, compress=compress, port=port,
+        extra_flags=extra_flags, iap_tunnel_args=iap_tunnel_args)
 
     if args.dry_run:
       log.out.Print(' '.join(cmd.Build(self.env)))
-      if tunnel_helper:
-        tunnel_helper.StopListener()
       return
 
     if args.plain or use_oslogin:
@@ -175,36 +167,23 @@ class BaseScpHelper(ssh_utils.BaseSSHCLIHelper):
           project)
 
     if keys_newly_added:
-      poller_tunnel_helper = None
-      if tunnel_helper:
-        poller_tunnel_helper = ssh_utils.CreateIapTunnelHelper(
-            args, instance_ref, instance, port=port)
-        poller_tunnel_helper.StartListener(accept_multiple_connections=True)
-      poller = ssh_utils.CreateSSHPoller(
-          remote, identity_file, options, poller_tunnel_helper, port=port)
+      poller = ssh_utils.CreateSSHPoller(remote, identity_file, options,
+                                         iap_tunnel_args, port=port)
 
       log.status.Print('Waiting for SSH key to propagate.')
       # TODO(b/35355795): Don't force_connect
       try:
         poller.Poll(self.env, force_connect=True)
       except retry.WaitException:
-        if tunnel_helper:
-          tunnel_helper.StopListener()
         raise ssh_utils.NetworkError()
-      finally:
-        if poller_tunnel_helper:
-          poller_tunnel_helper.StopListener()
 
-    if ip_type is ip.IpTypeEnum.INTERNAL and not tunnel_helper:
-      # The IAP Tunnel connection uses instance name and network interface name,
-      # so do not need to additionally verify the instance. Also, the
-      # SSHCommand used within the function does not support IAP Tunnels.
+    if ip_type is ip.IpTypeEnum.INTERNAL:
+      # This will never happen when IAP Tunnel is enabled, because ip_type is
+      # always EXTERNAL when IAP Tunnel is enabled, even if the instance has no
+      # external IP. IAP Tunnel doesn't need verification because it uses
+      # unambiguous identifiers for the instance.
       self.PreliminarilyVerifyInstance(instance.id, remote, identity_file,
                                        options)
 
-    try:
-      # Errors from the SCP command result in an ssh.CommandError being raised
-      cmd.Run(self.env, force_connect=True)
-    finally:
-      if tunnel_helper:
-        tunnel_helper.StopListener()
+    # Errors from the SCP command result in an ssh.CommandError being raised
+    cmd.Run(self.env, force_connect=True)
