@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@ from __future__ import unicode_literals
 
 from googlecloudsdk.command_lib.compute import completers as compute_completers
 from googlecloudsdk.command_lib.compute import flags as compute_flags
+from googlecloudsdk.command_lib.util import completers
 
 DEFAULT_LIST_FORMAT = """\
     table(
@@ -27,7 +28,7 @@ DEFAULT_LIST_FORMAT = """\
       creationTimestamp
     )"""
 
-ALPHA_LIST_FORMAT = """\
+BETA_LIST_FORMAT = """\
     table(
       name,
       type,
@@ -37,6 +38,8 @@ ALPHA_LIST_FORMAT = """\
       managed.domainStatus:format="yaml"
     )"""
 
+ALPHA_LIST_FORMAT = BETA_LIST_FORMAT
+
 
 class SslCertificatesCompleter(compute_completers.ListCommandCompleter):
 
@@ -47,27 +50,70 @@ class SslCertificatesCompleter(compute_completers.ListCommandCompleter):
         **kwargs)
 
 
-def SslCertificateArgument(required=True, plural=False):
+class GlobalSslCertificatesCompleter(compute_completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(GlobalSslCertificatesCompleter, self).__init__(
+        collection='compute.sslCertificates',
+        api_version='alpha',
+        list_command='alpha compute ssl-certificates list --global --uri',
+        **kwargs)
+
+
+class RegionSslCertificatesCompleter(compute_completers.ListCommandCompleter):
+
+  def __init__(self, **kwargs):
+    super(RegionSslCertificatesCompleter, self).__init__(
+        collection='compute.regionSslCertificates',
+        api_version='alpha',
+        list_command=
+        'alpha compute ssl-certificates list --filter=region:* --uri',
+        **kwargs)
+
+
+class SslCertificatesCompleterAlpha(completers.MultiResourceCompleter):
+
+  def __init__(self, **kwargs):
+    super(SslCertificatesCompleterAlpha, self).__init__(
+        completers=[
+            GlobalSslCertificatesCompleter, RegionSslCertificatesCompleter
+        ],
+        **kwargs)
+
+
+def SslCertificateArgument(required=True, plural=False, include_alpha=False):
   return compute_flags.ResourceArgument(
       resource_name='SSL certificate',
-      completer=SslCertificatesCompleter,
+      completer=SslCertificatesCompleterAlpha
+      if include_alpha else SslCertificatesCompleter,
       plural=plural,
       required=required,
-      global_collection='compute.sslCertificates')
+      global_collection='compute.sslCertificates',
+      regional_collection='compute.regionSslCertificates'
+      if include_alpha else None,
+      region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
+      if include_alpha else None)
 
 
-def SslCertificatesArgumentForOtherResource(resource, required=True):
+def SslCertificatesArgumentForOtherResource(resource,
+                                            required=True,
+                                            include_alpha=False):
   return compute_flags.ResourceArgument(
       name='--ssl-certificates',
       resource_name='ssl certificate',
-      completer=SslCertificatesCompleter,
+      completer=SslCertificatesCompleterAlpha
+      if include_alpha else SslCertificatesCompleter,
       plural=True,
       required=required,
       global_collection='compute.sslCertificates',
+      regional_collection='compute.regionSslCertificates'
+      if include_alpha else None,
+      region_explanation=compute_flags.REGION_PROPERTY_EXPLANATION
+      if include_alpha else None,
       short_help=('A reference to SSL certificate resources that are used for '
                   'server-side authentication.'),
       detailed_help="""\
-        References to at most 10 SSL certificate resources that are used for
+        References to at most 15 SSL certificate resources that are used for
         server-side authentication. The first SSL certificate in this list is
         considered the primary SSL certificate associated with the load
         balancer. The SSL certificate must exist and cannot be deleted while
