@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,9 @@
 """Flags and helpers for the compute ssl-policies commands."""
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import unicode_literals
+
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import completers as compute_completers
@@ -49,6 +51,7 @@ class SslPoliciesCompleter(compute_completers.ListCommandCompleter):
 
 def GetSslPolicyArgument(required=True, plural=False):
   """Returns the resource argument object for the SSL policy flag."""
+  # TODO(b/135122945): add name='SSL_POLICY' here and fix all help texts.
   return compute_flags.ResourceArgument(
       resource_name='SSL policy',
       completer=SslPoliciesCompleter,
