@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -213,9 +213,10 @@ def AddClusterAutoscalingFlags(parser, update_group=None, hidden=False):
 
   Args:
     parser: A given parser.
-    update_group: An optional group of mutually exclusive flag options
-        to which an --enable-autoscaling flag is added.
+    update_group: An optional group of mutually exclusive flag options to which
+      an --enable-autoscaling flag is added.
     hidden: If true, suppress help text for added options.
+
   Returns:
     Argument group for autoscaling flags.
   """
@@ -362,13 +363,15 @@ and memory limits to be specified.""",
   limits_group = group.add_mutually_exclusive_group()
   limits_group.add_argument(
       '--autoprovisioning-config-file',
-      type=arg_parsers.BufferedFileInput(),
+      type=arg_parsers.FileContents(),
       hidden=hidden,
       help="""\
 Path of the JSON/YAML file which contains information about the
-cluster's autoscaling configuration. Currently it only contains
-a list of resource limits of the cluster.
+cluster's node autoprovisioning configuration. Currently it contains
+a list of resource limits, identity defaults for autoprovisioning and node locations
+for autoprovisioning.
 
+Resource limits are specified in the field 'resourceLimits'.
 Each resource limits definition contains three fields:
 resourceType, maximum and minimum.
 Resource type can be "cpu", "memory" or an accelerator (e.g.
@@ -376,10 +379,24 @@ Resource type can be "cpu", "memory" or an accelerator (e.g.
 list to learn about available accelerator types.
 Maximum is the maximum allowed amount with the unit of the resource.
 Minimum is the minimum allowed amount with the unit of the resource.
+
+Identity default contains at most one of the below fields:
+serviceAccount: The Google Cloud Platform Service Account to be used by node VMs in
+autoprovisioined node pools. If not specified, the project default service account
+is used.
+scopes: A list of scopes be used by node instances in autoprovisioined node pools.
+Multiple scopes can be specified, separated by commas. For information on defaults,
+look at:
+https://cloud.google.com/sdk/gcloud/reference/container/clusters/create#--scopes
+
+Autoprovisioning locations is a set of zones where new node pools can be created by
+Autoprovisioning. Autoprovisioning locations are specified in the field
+'autoprovisioningLocations'. All zones must be in the same region as the cluster's
+master(s).
 """)
 
-  from_flags_group = limits_group.add_argument_group('Flags to configure '
-                                                     'resource limits:')
+  from_flags_group = limits_group.add_argument_group(
+      'Flags to configure autoprovisioned nodes')
   from_flags_group.add_argument(
       '--max-cpu',
       required=True,
@@ -454,6 +471,37 @@ accelerator-types list``` to learn about all available accelerator types.
 *count*::: (Required) The minimum number of accelerators
 to which the cluster can be scaled.
 """)
+  identity_group = from_flags_group.add_argument_group(
+      'Flags to specify identity for autoprovisioned nodes:', mutex=True)
+  identity_group.add_argument(
+      '--autoprovisioning-service-account',
+      type=str,
+      hidden=True,
+      help="""\
+The Google Cloud Platform Service Account to be used by node VMs in
+autoprovisioined node pools. If not specified, the project default
+service account is used.
+""")
+  identity_group.add_argument(
+      '--autoprovisioning-scopes',
+      type=arg_parsers.ArgList(),
+      metavar='SCOPE',
+      hidden=True,
+      help="""\
+The scopes be used by node instances in autoprovisioined node pools.
+Multiple scopes can be specified, separated by commas. For information
+on defaults, look at:
+https://cloud.google.com/sdk/gcloud/reference/container/clusters/create#--scopes
+""")
+  from_flags_group.add_argument(
+      '--autoprovisioning-locations',
+      hidden=True,
+      help="""\
+Set of zones where new node pools can be created by autoprovisioning.
+All zones must be in the same region as the cluster's master(s).
+Multiple locations can be specified, separated by commas.""",
+      metavar='ZONE',
+      type=arg_parsers.ArgList(min_length=1))
 
 
 def AddEnableBinAuthzFlag(parser, hidden=False):
@@ -562,6 +610,10 @@ def AddLocalSSDAndLocalSSDVolumeConfigsFlag(parser, for_node_pool=False,
 
 def AddLocalSSDVolumeConfigsFlag(parser, for_node_pool=False, help_text=''):
   """Adds a --local-ssd-volumes flag to the given parser."""
+  format_str = 'example_cluster'
+  if for_node_pool:
+    format_str = 'node-pool-1 --cluster=example-cluster'
+
   help_text += """\
 Adds the requested local SSDs on all nodes in default node-pool(s) in new cluster. Example:
 
@@ -577,8 +629,7 @@ Local SSDs have a fixed 375 GB capacity per device. The number of disks that
 can be attached to an instance is limited by the maximum number of disks
 available on a machine, which differs by compute zone. See
 https://cloud.google.com/compute/docs/disks/local-ssd for more information.
-""".format('node-pool-1 --cluster=example-cluster' if for_node_pool else
-           'example_cluster')
+""".format(format_str)
   count_validator = arg_parsers.RegexpValidator(
       r'^[1-8]$', 'Count must be a number between 1 and 8')
   type_validator = arg_parsers.RegexpValidator(
@@ -1088,7 +1139,7 @@ def AddIPAliasFlags(parser):
   parser.add_argument(
       '--enable-ip-alias',
       action='store_true',
-      default=None,
+      default=True,
       help="""\
 Enable use of alias IPs (https://cloud.google.com/compute/docs/alias-ip/)
 for pod IPs. This will create two secondary ranges, one for the pod IPs
@@ -1259,22 +1310,21 @@ def AddWorkloadMetadataFromNodeFlag(parser, hidden=False):
     parser: A given parser.
     hidden: Whether or not to hide the help text.
   """
-  help_text = """\
-Sets the node metadata option for workload metadata configuration. This feature
-is scheduled to be deprecated in the future and later removed.
-"""
 
   parser.add_argument(
       '--workload-metadata-from-node',
       default=None,
       choices={
-          'SECURE': 'Prevents workloads not in hostNetwork from accessing '
+          'SECURE': 'Prevents pods not in hostNetwork from accessing '
                     'certain VM metadata, specifically kube-env, which '
                     'contains Kubelet credentials, and the instance identity '
                     'token. This is a temporary security solution available '
                     'while the bootstrapping process for cluster nodes is '
-                    'being redesigned with significant security improvements.',
-          'EXPOSED': 'Exposes all VM metadata to workloads.',
+                    'being redesigned with significant security improvements. '
+                    'This feature is scheduled to be deprecated in the future '
+                    'and later removed.',
+          'EXPOSED': "Pods running in this nodepool have access to the node's "
+                     "underlying Compute Engine Metadata Server.",
           'GKE_METADATA_SERVER':
               'Run the Kubernetes Engine Metadata Server on this node. The Kubernetes '
               'Engine Metadata Server exposes a metadata API to workloads that is '
@@ -1285,7 +1335,7 @@ is scheduled to be deprecated in the future and later removed.
       },
       type=lambda x: x.upper(),
       hidden=hidden,
-      help=help_text)
+      help='Type of metadata server available to pods running in the nodepool.')
 
 
 def AddTagOrDigestPositional(parser,
@@ -1344,22 +1394,14 @@ Multiple locations can be specified, separated by commas. For example:
 """)
 
 
-def AddLoggingServiceFlag(parser, enable_kubernetes):
+def AddLoggingServiceFlag(parser):
   """Adds a --logging-service flag to the parser.
 
   Args:
     parser: A given parser.
-    enable_kubernetes: Mention Kubernetes-native resource model in help string
   """
   help_str = """\
 Logging service to use for the cluster. Options are:
-"logging.googleapis.com" (the Google Cloud Logging service),
-"none" (logs will not be exported from the cluster)
-"""
-
-  if enable_kubernetes:
-    help_str = """\
-Logging service to use for the cluster. Options are:
 "logging.googleapis.com/kubernetes" (the Google Cloud Logging
 service with Kubernetes-native resource model enabled),
 "logging.googleapis.com" (the Google Cloud Logging service),
@@ -1369,23 +1411,15 @@ service with Kubernetes-native resource model enabled),
   parser.add_argument('--logging-service', help=help_str)
 
 
-def AddMonitoringServiceFlag(parser, enable_kubernetes):
+def AddMonitoringServiceFlag(parser):
   """Adds a --monitoring-service flag to the parser.
 
   Args:
     parser: A given parser.
-    enable_kubernetes: Mention Kubernetes-native resource model in help string
   """
 
   help_str = """\
 Monitoring service to use for the cluster. Options are:
-"monitoring.googleapis.com" (the Google Cloud Monitoring service),
-"none" (no metrics will be exported from the cluster)
-"""
-
-  if enable_kubernetes:
-    help_str = """\
-Monitoring service to use for the cluster. Options are:
 "monitoring.googleapis.com/kubernetes" (the Google Cloud
 Monitoring service with Kubernetes-native resource model enabled),
 "monitoring.googleapis.com" (the Google Cloud Monitoring service),
@@ -1395,38 +1429,18 @@ Monitoring service with Kubernetes-native resource model enabled),
   parser.add_argument('--monitoring-service', help=help_str)
 
 
-def AddNodeIdentityFlags(parser, example_target, new_behavior=True):
+def AddNodeIdentityFlags(parser, example_target):
   """Adds node identity flags to the given parser.
 
-  Node identity flags are --scopes, --[no-]enable-cloud-endpoints (deprecated),
-  and --service-account.  --service-account is mutually exclusive with the
-  others.  --[no-]enable-cloud-endpoints is not allowed if property
-  container/new_scopes_behavior is set to true, and is removed completely if
-  new_behavior is set to true.
+  Node identity flags are --scopes and --service-account.
 
   Args:
     parser: A given parser.
     example_target: the target for the command, e.g. mycluster.
-    new_behavior: Use new (alpha & beta) behavior: remove
-    --[no-]enable-cloud-endpoints.
   """
   node_identity_group = parser.add_group(
-      mutex=True, help='Options to specify the node identity.')
+      help='Options to specify the node identity.')
   scopes_group = node_identity_group.add_group(help='Scopes options.')
-
-  if new_behavior:
-    track_help = """
-Unless container/new_scopes_behavior property is true, compute-rw and storage-ro
-are always added, even if not explicitly specified, and --enable-cloud-endpoints
-(by default) adds service-control and service-management scopes.
-
-If container/new_scopes_behavior property is true, none of the above scopes are
-added (though storage-ro, service-control, and service-management are all
-included in the default scopes.  In a future release, this will be the default
-behavior.
-"""
-  else:
-    track_help = ''
   scopes_group.add_argument(
       '--scopes',
       type=arg_parsers.ArgList(),
@@ -1443,37 +1457,11 @@ Multiple SCOPEs can be specified, separated by commas. `logging-write`
 and/or `monitoring` are added unless Cloud Logging and/or Cloud Monitoring
 are disabled (see `--enable-cloud-logging` and `--enable-cloud-monitoring`
 for more information).
-{track_help}
 {scopes_help}
 """.format(
     example_target=example_target,
-    track_help=track_help,
     scopes_help=compute_constants.ScopesHelp()))
 
-  cloud_endpoints_help_text = """\
-Automatically enable Google Cloud Endpoints to take advantage of API management
-features by adding service-control and service-management scopes.
-
-If `--no-enable-cloud-endpoints` is set, remove service-control and
-service-management scopes, even if they are implicitly (via default) or
-explicitly set via `--scopes`.
-
-`--[no-]enable-cloud-endpoints` is not allowed if
-`container/new_scopes_behavior` property is set to true.
-"""
-  scopes_group.add_argument(
-      '--enable-cloud-endpoints',
-      action=actions.DeprecationAction(
-          '--[no-]enable-cloud-endpoints',
-          warn='Flag --[no-]enable-cloud-endpoints is deprecated and will be '
-          'removed in a future release.  Scopes necessary for Google Cloud '
-          'Endpoints are now included in the default set and may be '
-          'excluded using --scopes.',
-          removed=new_behavior,
-          action='store_true'),
-      default=True,
-      help=cloud_endpoints_help_text)
-
   sa_help_text = (
       'The Google Cloud Platform Service Account to be used by the node VMs. '
       'If a service account is specified, the cloud-platform and '
@@ -1495,20 +1483,6 @@ def AddClusterNodeIdentityFlags(parser):
   AddNodeIdentityFlags(parser, example_target='example-cluster')
 
 
-def AddDeprecatedClusterNodeIdentityFlags(parser):
-  """Adds node identity flags to the given parser.
-
-  This is a wrapper around AddNodeIdentityFlags for [alpha|beta] cluster, as it
-  provides example-cluster as the example and uses non-deprecated scopes
-  behavior.
-
-  Args:
-    parser: A given parser.
-  """
-  AddNodeIdentityFlags(
-      parser, example_target='example-cluster', new_behavior=False)
-
-
 def AddNodePoolNodeIdentityFlags(parser):
   """Adds node identity flags to the given parser.
 
@@ -1522,21 +1496,6 @@ def AddNodePoolNodeIdentityFlags(parser):
       parser, example_target='node-pool-1 --cluster=example-cluster')
 
 
-def AddDeprecatedNodePoolNodeIdentityFlags(parser):
-  """Adds node identity flags to the given parser.
-
-  This is a wrapper around AddNodeIdentityFlags for (GA) node-pools, as it
-  provides node-pool-1 as the example and uses non-deprecated scopes behavior.
-
-  Args:
-    parser: A given parser.
-  """
-  AddNodeIdentityFlags(
-      parser,
-      example_target='node-pool-1 --cluster=example-cluster',
-      new_behavior=False)
-
-
 def AddAddonsFlagsWithOptions(parser, addon_options):
   """Adds the --addons flag to the parser with the given addon options."""
   parser.add_argument(
@@ -1545,10 +1504,10 @@ def AddAddonsFlagsWithOptions(parser, addon_options):
       metavar='ADDON',
       # TODO(b/65264376): Replace the doc link when a better doc is ready.
       help="""\
-Default set of addons includes {0}. Addons
+Addons
 (https://cloud.google.com/kubernetes-engine/reference/rest/v1/projects.zones.clusters#AddonsConfig)
 are additional Kubernetes cluster components. Addons specified by this flag will
-be enabled. The others will be disabled.
+be enabled. The others will be disabled. Default addons: {0}.
 """.format(', '.join(api_adapter.DEFAULT_ADDONS)))
 
 
@@ -1620,8 +1579,7 @@ def AddTpuFlags(parser, hidden=False, enable_tpu_service_networking=False):
       help="""\
 Enable Cloud TPUs for this cluster.
 
-Can not be specified unless `--enable-kubernetes-alpha` and `--enable-ip-alias`
-are also specified.
+Can not be specified unless `--enable-ip-alias` is also specified.
 """)
 
   group = tpu_group
@@ -1761,17 +1719,6 @@ your cluster size.'
       help=help_text)
 
 
-# TODO(b/110368338): Drop this warning when changing the default value of the
-# flag.
-def WarnForUnspecifiedIpAllocationPolicy(args):
-  if not args.IsSpecified('enable_ip_alias'):
-    log.warning(
-        'Currently VPC-native is not the default mode during cluster creation. '
-        'In the future, this will become the default mode and can be disabled '
-        'using `--no-enable-ip-alias` flag. Use `--[no-]enable-ip-alias` flag '
-        'to suppress this warning.')
-
-
 def WarnForNodeModification(args, enable_autorepair):
   if (args.image_type or '').lower() != 'ubuntu':
     return
@@ -1783,10 +1730,7 @@ def WarnForNodeModification(args, enable_autorepair):
                 'DaemonSet.')
 
 
-def WarnForAutoUpgrade(args):
-  if not args.IsSpecified('enable_autoupgrade'):
-    log.warning(util.WARN_AUTOUPGRADE_ENABLED_BY_DEFAULT)
-
+def WarnForNodeVersionAutoUpgrade(args):
   if args.IsSpecified('node_version') and args.enable_autoupgrade:
     log.warning(util.WARN_NODE_VERSION_WITH_AUTOUPGRADE_ENABLED)
 
@@ -1890,13 +1834,26 @@ the Cloud project containing the cluster, `PROJECT_NAME.svc.id.goog`.
 """)
 
 
-def AddResourceUsageExportFlags(parser, add_clear_flag=False, hidden=False):
+def AddWorkloadIdentityUpdateFlags(parser):
+  # TODO(b/126751755): Once we have a beta docs page, update the documentation
+  # link in the help text.
+  parser.add_argument(
+      '--disable-workload-identity',
+      default=False,
+      action='store_true',
+      hidden=True,
+      help="""\
+Disable Workload Identity on the cluster.
+""")
+
+
+def AddResourceUsageExportFlags(parser, is_update=False, hidden=False):
   """Adds flags about exporting cluster resource usage to BigQuery."""
 
   group = parser.add_group(
       "Exports cluster's usage of cloud resources",
       hidden=hidden)
-  if add_clear_flag:
+  if is_update:
     group.is_mutex = True
     group.add_argument(
         '--clear-resource-usage-bigquery-dataset',
@@ -1923,7 +1880,7 @@ Example:
       hidden=hidden,
       help=dataset_help_text)
 
-  network_egress_help_text = """`
+  network_egress_help_text = """\
 Enable network egress metering on this cluster.
 
 When enabled, a DaemonSet is deployed into the cluster. Each DaemonSet pod
@@ -1937,8 +1894,40 @@ Network egress metering is disabled if this flag is omitted, or when
       '--enable-network-egress-metering',
       action='store_true',
       default=None,
+      hidden=hidden,
       help=network_egress_help_text)
 
+  resource_consumption_help_text = """\
+Enable resource consumption metering on this cluster.
+
+When enabled, a table will be created in the specified BigQuery dataset to store
+resource consumption data. The resulting table can be joined with the resource
+usage table or with BigQuery billing export.
+
+Resource consumption metering is enabled unless `--no-enable-resource-
+consumption-metering` is set.
+"""
+
+  if is_update:
+    resource_consumption_help_text = """\
+Enable resource consumption metering on this cluster.
+
+When enabled, a table will be created in the specified BigQuery dataset to store
+resource consumption data. The resulting table can be joined with the resource
+usage table or with BigQuery billing export.
+
+To disable resource consumption metering, set `--no-enable-resource-consumption-
+metering`. If this flag is omitted, then resource consumption metering will
+remain enabled or disabled depending on what is already configured for this
+cluster.
+"""
+  group.add_argument(
+      '--enable-resource-consumption-metering',
+      action='store_true',
+      default=None,
+      hidden=hidden,
+      help=resource_consumption_help_text)
+
 
 def AddEnablePrivateIpv6AccessFlag(parser, hidden=False):
   """Adds --enable-private-ipv6-access flag to the parser.
@@ -1970,6 +1959,32 @@ This is currently only available on Alpha clusters, specified by using
       action='store_true')
 
 
+def AddEnableIntraNodeVisibilityFlag(parser, hidden=False):
+  """Adds --enable-intra-node-visibility flag to the parser.
+
+  When enabled, the intra-node traffic is visible to VPC network.
+
+  Args:
+    parser: A given parser.
+    hidden: If true, suppress help text for added options.
+  """
+  parser.add_argument(
+      '--enable-intra-node-visibility',
+      default=None,
+      hidden=hidden,
+      action='store_true',
+      help="""\
+Enable Intra-node visibility for this cluster.
+
+Enabling intra-node visibility makes your intra-node pod to pod traffic
+visible to the networking fabric. With this feature, you can use VPC flow
+logging or other VPC features for intra-node traffic.
+
+This is a beta feature, enabling it on existing cluster causes the cluster
+master and the cluster nodes to restart, which might cause disruption.
+""")
+
+
 def AddVerticalPodAutoscalingFlag(parser, hidden=False):
   """Adds vertical pod autoscaling related flag to the parser.
 
@@ -1988,7 +2003,6 @@ def AddVerticalPodAutoscalingFlag(parser, hidden=False):
       action='store_true')
 
 
-# TODO(b/112194849): Explain limitation to the sandbox pods and the nodes.
 def AddSandboxFlag(parser, hidden=False):
   """Adds a --sandbox flag to the given parser.
 
@@ -2009,7 +2023,7 @@ def AddSandboxFlag(parser, hidden=False):
       help="""\
 Enables the requested sandbox on all nodes in the node-pool. Example:
 
-  $ {command} node-pool-1 --cluster=example-cluster --sandbox type=gvisor
+  $ {command} node-pool-1 --cluster=example-cluster --sandbox="type=gvisor"
 
 The only supported type is 'gvisor'.
       """)
@@ -2211,14 +2225,205 @@ def AddMetadataFlags(parser):
       metavar='KEY=LOCAL_FILE_PATH')
 
 
-def AddEnableShieldedContainersFlags(parser):
-  """Adds a --enable-shielded-containers flag to the given parser."""
+def AddEnableShieldedNodesFlags(parser):
+  """Adds a --enable-shielded-nodes flag to the given parser."""
   help_text = """\
-Enable shielded containers for this cluster. Enabling shielded containers
-will enable a more secure Node credential bootstrapping implementation.
+Enable Shielded Nodes for this cluster. Enabling Shielded Nodes will enable a
+more secure Node credential bootstrapping implementation.
 """
   parser.add_argument(
-      '--enable-shielded-containers',
+      '--enable-shielded-nodes',
       action='store_true',
+      default=None,
       help=help_text,
       hidden=True)
+
+
+def AddSurgeUpgradeFlag(parser, hidden=False, for_node_pool=False):
+  """Adds surge upgrade related flag to the parser.
+
+  Args:
+    parser: A given parser.
+    hidden: Whether or not to hide the help text.
+    for_node_pool: Whether this flag is for a node pool.
+  """
+
+  if for_node_pool:
+    max_surge_help = """\
+Number of extra (surge) nodes to be created on each upgrade of the node pool.
+
+Specifies the number of extra (surge) nodes to be created during this node
+pool's upgrades. For example, running the following command will result in
+creating an extra node each time the node pool is upgraded:
+
+  $ {command} node-pool-1 --cluster=example-cluster --max-surge-upgrade 1
+"""
+  else:
+    max_surge_help = """\
+Number of extra (surge) nodes to be created on each upgrade of a node pool.
+
+Specifies the number of extra (surge) nodes to be created during this node
+pool's upgrades. For example, running the following command will result in
+creating an extra node each time the node pool is upgraded:
+
+  $ {command} example-cluster --max-surge-upgrade 1
+"""
+  parser.add_argument(
+      '--max-surge-upgrade',
+      type=int,
+      default=0,
+      help=max_surge_help,
+      hidden=hidden)
+
+
+def AddMaxUnavailableUpgradeFlag(parser, hidden=False, for_node_pool=False):
+  """Adds --max-unavailable-upgrade flag to the parser.
+
+  Args:
+    parser: A given parser.
+    hidden: Whether or not to hide the help text.
+    for_node_pool: Whether this flag is for a node pool.
+  """
+
+  if for_node_pool:
+    max_unavailable_upgrade_help = """\
+Number of nodes that can be unavailable at the same time on each upgrade of the node pool.
+
+Specifies the number of nodes that can be unavailable at the same time during this node
+pool's upgrades. For example, running the following command will result in
+having 3 nodes being upgraded in parallel (1 + 2), but keeping always at least 3 (5 - 2)
+available each time the node pool is upgraded:
+
+  $ {command} node-pool-1 --cluster=example-cluster --num-nodes=5 --max-surge-upgrade 1 --max-unavailable-upgrade=2
+"""
+  else:
+    max_unavailable_upgrade_help = """\
+Number of nodes that can be unavailable at the same time on each upgrade of a
+node pool.
+
+Specifies the number of nodes that can be unavailable at the same time while
+this node pool is being upgraded. For example, running the following command
+will result in having 3 nodes being upgraded in parallel (1 + 2), but keeping
+always at least 3 (5 - 2) available each time the node pool is upgraded:
+
+   $ {command} example-cluster --num-nodes=5 --max-surge-upgrade 1 \
+     --max-unavailable-upgrade=2
+"""
+  parser.add_argument(
+      '--max-unavailable-upgrade',
+      type=int,
+      default=1,
+      help=max_unavailable_upgrade_help,
+      hidden=hidden)
+
+
+def AddLinuxSysctlFlags(parser, for_node_pool=False):
+  """Adds Linux sysctl flag to the given parser."""
+  if for_node_pool:
+    help_text = """\
+Linux kernel parameters to be applied to all nodes in the new node-pool as well
+as the pods running on the nodes.
+
+Example:
+
+  $ {command} node-pool-1 --linux-sysctls="net.core.somaxconn=1024,net.ipv4.tcp_rmem=4096 87380 6291456"
+"""
+  else:
+    help_text = """\
+Linux kernel parameters to be applied to all nodes in the new cluster's default
+node pool as well as the pods running on the nodes.
+
+Example:
+
+  $ {command} example-cluster --linux-sysctls="net.core.somaxconn=1024,net.ipv4.tcp_rmem=4096 87380 6291456"
+"""
+  parser.add_argument(
+      '--linux-sysctls',
+      type=arg_parsers.ArgDict(min_length=1),
+      default={},
+      help=help_text,
+      metavar='KEY=VALUE',
+      action=arg_parsers.StoreOnceAction)
+
+
+def AddDisableDefaultSnatFlag(parser, for_cluster_create=False):
+  """Adds disable-default-snat flag to the parser.
+
+  Args:
+    parser: A given parser.
+    for_cluster_create: Whether the flag is for cluster creation.
+  """
+
+  if for_cluster_create:
+    help_text = """\
+Disable default source NAT rules applied in cluster nodes.
+
+By default, network traffic sending from Pods to outside of VPC will get
+masqueraded by the node external IP. When this flag is set, no default sNAT
+will be enforced on the cluster. The flag must be set if the cluster uses
+privately used public IPs.
+
+Must be used in conjunction with `--enable-ip-alias` and `--enable-private-nodes`.
+"""
+  else:
+    help_text = """\
+Disable default source NAT rules applied in cluster nodes.
+
+By default, network traffic sending from Pods to outside of VPC will get
+masqueraded by the node external IP. When this flag is set, no default sNAT
+will be enforced on the cluster. The flag must be set if the cluster uses
+privately used public IPs.
+
+Can only be used with private cluster in VPC Native network mode.
+"""
+  parser.add_argument(
+      '--disable-default-snat',
+      default=(False if for_cluster_create else None),
+      action='store_true',
+      help=help_text)
+
+
+def AddNodePoolLocationsFlag(parser, for_create=False):
+  """Adds a --node-locations flag for node pool to parser."""
+  if for_create:
+    help_text = """
+The set of zones in which the NodePool's nodes should be located.
+
+Multiple locations can be specified, separated by commas. For example:
+
+  $ {command} node-pool-1 --node-locations=us-central1-a,us-central1-b"""
+  else:
+    help_text = """\
+Set of zones in which the NodePool's nodes should be located.
+Changing the locations for a node pool will result in nodes being either created or removed
+from the node pool, depending on whether locations are being added or removed.
+
+Multiple locations can be specified, separated by commas. For example:
+
+  $ {command} node-pool-1 --node-locations=us-central1-a,us-central1-b"""
+  parser.add_argument(
+      '--node-locations',
+      type=arg_parsers.ArgList(min_length=1),
+      metavar='ZONE',
+      hidden=True,
+      help=help_text)
+
+
+def AddShieldedInstanceFlags(parser):
+  """Adds Shielded Instance flags to the given parser."""
+  secure_boot_help = """\
+      The instance will boot with secure boot enabled.
+      """
+  parser.add_argument(
+      '--shielded-secure-boot', action='store_true', help=secure_boot_help)
+
+  integrity_monitoring_help = """\
+      Enables monitoring and attestation of the boot integrity of the
+      instance. The attestation is performed against the integrity policy
+      baseline. This baseline is initially derived from the implicitly
+      trusted boot image when the instance is created.
+      """
+  parser.add_argument(
+      '--shielded-integrity-monitoring',
+      action='store_true',
+      help=integrity_monitoring_help)
