@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2019 Google Inc. All Rights Reserved.
+# Copyright 2019 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,9 +18,11 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+from googlecloudsdk.command_lib.filestore import locations_util
 from googlecloudsdk.core import properties
 
 INSTANCE_NAME_TEMPLATE = 'projects/{}/locations/{}/instances/{}'
+SNAPSHOT_NAME_TEMPLATE = 'projects/{}/locations/{}/snapshots/{}'
 
 
 def AddInstanceNameToRequest(ref, args, req):
@@ -30,3 +32,13 @@ def AddInstanceNameToRequest(ref, args, req):
   req.snapshot.sourceInstance = INSTANCE_NAME_TEMPLATE.format(
       project, args.instance_zone, args.instance)
   return req
+
+
+def AddSnapshotNameToRequest(ref, args, req):
+  """Python hook for yaml commands to process the source snapshot name."""
+  project = properties.VALUES.core.project.Get(required=True)
+  location = (args.source_snapshot_region or
+              locations_util.GetRegionFromZone(ref.locationsId))
+  req.restoreInstanceRequest.sourceSnapshot = SNAPSHOT_NAME_TEMPLATE.format(
+      project, location, args.source_snapshot)
+  return req
