@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Provides common arguments for the ML Engine command surface."""
+"""Provides common arguments for the AI Platform command surface."""
 
 from __future__ import absolute_import
 from __future__ import division
@@ -33,8 +33,6 @@ from googlecloudsdk.calliope.concepts import concepts
 from googlecloudsdk.command_lib.iam import completers as iam_completers
 from googlecloudsdk.command_lib.ml_engine import models_util
 from googlecloudsdk.command_lib.util.apis import arg_utils
-from googlecloudsdk.command_lib.util.args import repeated
-from googlecloudsdk.command_lib.util.args import update_util
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
@@ -220,17 +218,6 @@ Path to Python archives used for training. These can be local paths
 Storage bucket given by `--staging-bucket`, or Cloud Storage URLs
 (`gs://bucket-name/path/to/package.tar.gz`).
 """)
-MACHINE_TYPE = base.Argument(
-    '--machine-type',
-    required=False,
-    choices=['mls1-c1-m2', 'mls1-c4-m2'],
-    help="""\
-Type of machine on which to serve the model. Currently only applies to
-online prediction. Currently supported machine_types are:
-
-* `mls1-c1-m2` - A virtual machine with 1 core and 2 Gb RAM (default).
-* `mls1-c4-m2` - A virtual machine with 4 core and 2 Gb RAM.
-""")
 
 
 SERVICE_ACCOUNT = base.Argument(
@@ -267,10 +254,11 @@ def GetJobDirFlag(upload_help=True, allow_local=False):
 needed for training.
 
 This path will be passed to your TensorFlow program as the `--job_dir` command-line
-arg. The benefit of specifying this field is that Cloud ML Engine will validate
+arg. The benefit of specifying this field is that AI Platform will validate
 the path for use in training.
-""".format(dir_type=('Google Cloud Storage path' +
-                     (' or local_directory' if allow_local else '')))
+""".format(
+    dir_type=('Google Cloud Storage path' +
+              (' or local_directory' if allow_local else '')))
   if upload_help:
     help_ += """\
 
@@ -282,7 +270,7 @@ will be used instead.
     type_ = str
   else:
     type_ = functools.partial(storage_util.ObjectReference.FromArgument,
-                              allow_empty_object=True)  # pytype: disable=wrong-arg-types
+                              allow_empty_object=True)
   return base.Argument('--job-dir', type=type_, help=help_)
 
 
@@ -305,7 +293,7 @@ VERSION_NAME = base.Argument('version', help='Name of the model version.')
 RUNTIME_VERSION = base.Argument(
     '--runtime-version',
     help=(
-        'Google Cloud ML Engine runtime version for this job. Defaults '
+        'AI Platform runtime version for this job. Defaults '
         'to a stable version, which is defined in documentation along '
         'with the list of supported versions: '
         'https://cloud.google.com/ml-engine/docs/tensorflow/runtime-version-list'  # pylint: disable=line-too-long
@@ -471,30 +459,29 @@ def GetVersionResourceSpec():
 
 
 def AddVersionResourceArg(parser, verb):
-  """Add a resource argument for a Cloud ML Engine version."""
+  """Add a resource argument for an AI Platform version."""
   concept_parsers.ConceptParser.ForResource(
       'version',
       GetVersionResourceSpec(),
-      'The Cloud ML Engine model {}.'.format(verb),
+      'The AI Platform model {}.'.format(verb),
       required=True).AddToParser(parser)
 
 
 def AddUserCodeArgs(parser):
   """Add args that configure user prediction code."""
-  user_code_group = base.ArgumentGroup(
-      help="""\
+  user_code_group = base.ArgumentGroup(help="""\
           Configure user code in prediction.
 
-          Cloud ML Engine allows a model to have user-provided prediction
+          AI Platform allows a model to have user-provided prediction
           code; these options configure that code.
           """)
   user_code_group.AddArgument(base.Argument(
-      '--model-class',
+      '--prediction-class',
       help="""\
-          The fully-qualified name of the custom Model class in the package
+          The fully-qualified name of the custom prediction class in the package
           provided for custom prediction.
 
-          For example, `--model-class my_package.SequenceModel`.
+          For example, `--prediction-class my_package.SequenceModel`.
           """))
   user_code_group.AddArgument(base.Argument(
       '--package-uris',
@@ -507,27 +494,6 @@ def AddUserCodeArgs(parser):
   user_code_group.AddToParser(parser)
 
 
-def AddUserCodeUpdateArgs(parser):
-  """Add args that configure user prediction code."""
-  user_code_group = parser.add_group(
-      help="""\
-          Configure user code in prediction.
-
-          Cloud ML Engine allows a model to have user-provided prediction
-          code; these options configure that code.
-          """)
-  repeated.AddPrimitiveArgs(
-      user_code_group, 'version', 'package-uris',
-      'user-supplied packages to use for prediction',
-      additional_help=('The values should be given as a comma-separated list '
-                       'of Google Cloud Storage URIs (`gs://...`).'))
-  update_util.AddClearableField(
-      user_code_group, 'model-class', 'custom Model class', 'version',
-      ('The fully-qualified name of the custom Model class in the package '
-       'provided for custom prediction.\n\n'
-       'For example, `my_package.SequenceModel`.'))
-
-
 def GetAcceleratorFlag():
   return base.Argument(
       '--accelerator',
@@ -716,10 +682,10 @@ def GetWorkerMachineConfig():
             'job\'s worker nodes.'))
 
   machine_count = base.Argument(
-      '--worker-server-count',
+      '--worker-count',
       type=arg_parsers.BoundedInt(1, sys.maxsize, unlimited=True),
       required=True,
-      help=('The number of worker nodes to use for the training job.'))
+      help='The number of worker nodes to use for the training job.')
 
   machine_type_group = base.ArgumentGroup(
       required=False,
@@ -744,3 +710,12 @@ def GetWorkerImageUri():
       help=('Docker image to run on each worker node. '
             'This image must be in Google Container Registry. If not '
             'specified, the value of `master-image-uri` is used.'))
+
+
+def AddMachineTypeFlagToParser(parser):
+  base.Argument(
+      '--machine-type',
+      help="""\
+Type of machine on which to serve the model. Currently only applies to online prediction. For available machine types,
+see https://cloud.google.com/ml-engine/docs/tensorflow/online-predict#machine-types.
+""").AddToParser(parser)
