@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,25 +20,27 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 import abc
+import base64
 import contextlib
+import functools
+import os
 import re
 import ssl
 import sys
+import tempfile
 from googlecloudsdk.api_lib.run import gke
+from googlecloudsdk.api_lib.run import global_methods
 from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
 from googlecloudsdk.command_lib.run import flags
 
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.util import files
 
 import six
 from six.moves.urllib import parse as urlparse
 
 
-_SERVERLESS_API_NAME = 'run'
-_SERVERLESS_API_VERSION = 'v1alpha1'
-
-
 @contextlib.contextmanager
 def _OverrideEndpointOverrides(override):
   """Context manager to override the Cloud Run endpoint overrides for a while.
@@ -66,11 +68,11 @@ class ConnectionInfo(six.with_metaclass(abc.ABCMeta)):
 
   @property
   def api_name(self):
-    return _SERVERLESS_API_NAME
+    return global_methods.SERVERLESS_API_NAME
 
   @property
   def api_version(self):
-    return _SERVERLESS_API_VERSION
+    return global_methods.SERVERLESS_API_VERSION
 
   @abc.abstractmethod
   def Connect(self):
@@ -84,6 +86,10 @@ class ConnectionInfo(six.with_metaclass(abc.ABCMeta)):
   def ns_label(self):
     pass
 
+  @abc.abstractproperty
+  def supports_one_platform(self):
+    pass
+
   @abc.abstractproperty
   def location_label(self):
     pass
@@ -124,17 +130,49 @@ def _CheckTLSSupport():
             min_required_version))
 
 
-class _GKEConnectionContext(ConnectionInfo):
-  """Context manager to connect to the GKE Cloud Run add-in."""
+class _ClusterConnectionContext(ConnectionInfo):
+  """Context manager to connect to a generic cluster."""
 
-  def __init__(self, cluster_ref):
-    super(_GKEConnectionContext, self).__init__()
-    self.cluster_ref = cluster_ref
+  def __init__(self, conn_info_getter):
+    super(_ClusterConnectionContext, self).__init__()
+    self.conn_info_getter = conn_info_getter
+    self.region = None
 
   @property
   def ns_label(self):
     return 'namespace'
 
+  @property
+  def operator(self):
+    return 'cluster'
+
+  @abc.abstractproperty
+  def location_label(self):
+    pass
+
+  @contextlib.contextmanager
+  def Connect(self):
+    _CheckTLSSupport()
+    with self.conn_info_getter() as (ip, ca_certs):
+      self.ca_certs = ca_certs
+      with gke.MonkeypatchAddressChecking('kubernetes.default', ip) as endpoint:
+        self.endpoint = 'https://{}/'.format(endpoint)
+        with _OverrideEndpointOverrides(self.endpoint):
+          yield self
+
+  @property
+  def supports_one_platform(self):
+    return False
+
+
+class _GKEConnectionContext(_ClusterConnectionContext):
+  """Context manager to connect to the GKE Cloud Run add-in."""
+
+  def __init__(self, cluster_ref):
+    super(_GKEConnectionContext, self).__init__(
+        functools.partial(gke.ClusterConnectionInfo, cluster_ref))
+    self.cluster_ref = cluster_ref
+
   @property
   def operator(self):
     return 'Cloud Run on GKE'
@@ -144,15 +182,74 @@ class _GKEConnectionContext(ConnectionInfo):
     return ' of cluster [{{{{bold}}}}{}{{{{reset}}}}]'.format(
         self.cluster_ref.Name())
 
+
+class _KubeconfigConnectionContext(_ClusterConnectionContext):
+  """Context manager to connect to a cluster defined in a Kubeconfig file."""
+
+  def __init__(self, kubeconfig, context=None):
+    super(_KubeconfigConnectionContext, self).__init__(
+        self._LoadClusterDetails)
+    self.config = kubeconfig
+    self.context = context
+
+  @property
+  def location_label(self):
+    return ' of cluster [{{{{bold}}}}{}{{{{reset}}}}]'.format(
+        self.cluster['name'])
+
   @contextlib.contextmanager
-  def Connect(self):
-    _CheckTLSSupport()
-    with gke.ClusterConnectionInfo(self.cluster_ref) as (ip, ca_certs):
-      self.ca_certs = ca_certs
-      with gke.MonkeypatchAddressChecking('kubernetes.default', ip) as endpoint:
-        self.endpoint = 'https://{}/'.format(endpoint)
-        with _OverrideEndpointOverrides(self.endpoint):
-          yield self
+  def _LoadClusterDetails(self):
+    """Get the current cluster and its connection info from the kubeconfig.
+
+    Yields:
+      A tuple of (endpoint, ca_certs), where endpoint is the ip address
+      of the GKE master, and ca_certs is the absolute path of a temporary file
+      (lasting the life of the python process) holding the ca_certs to connect
+      to the GKE cluster.
+    Raises:
+      flags.KubeconfigError: if the config file has missing keys or values.
+    """
+    try:
+      if self.context:
+        curr_ctx_name = self.context
+      else:
+        curr_ctx_name = self.config['current-context']
+      curr_ctx = next((c for c in self.config['contexts']
+                       if c['name'] == curr_ctx_name), None)
+      if not curr_ctx:
+        raise flags.KubeconfigError(
+            'Count not find context [{}] in kubeconfig.'.format(
+                curr_ctx_name))
+
+      self.cluster = next((c for c in self.config['clusters']
+                           if c['name'] == curr_ctx['context']['cluster']),
+                          None)
+      if not self.cluster:
+        raise flags.KubeconfigError(
+            'Could not find cluster [{}] specified by context [{}] in '
+            'kubeconfig.'.format(
+                curr_ctx['context']['cluster'], curr_ctx_name))
+
+      ca_data = self.cluster['cluster']['certificate-authority-data']
+      endpoint = self.cluster['cluster']['server'].replace('https://', '')
+    except KeyError as e:
+      raise flags.KubeconfigError('Missing key `{}` in kubeconfig.'.format(
+          e.args[0]))
+
+    fd, filename = tempfile.mkstemp()
+    os.close(fd)
+    files.WriteBinaryFileContents(
+        filename, base64.b64decode(ca_data), private=True)
+    try:
+      yield endpoint, filename
+    finally:
+      os.remove(filename)
+
+
+def DeriveRegionalEndpoint(endpoint, region):
+  scheme, netloc, path, params, query, fragment = urlparse.urlparse(endpoint)
+  netloc = '{}-{}'.format(region, netloc)
+  return urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
 
 
 class _RegionalConnectionContext(ConnectionInfo):
@@ -177,16 +274,17 @@ class _RegionalConnectionContext(ConnectionInfo):
 
   @contextlib.contextmanager
   def Connect(self):
-    global_endpoint = apis.GetEffectiveApiEndpoint(_SERVERLESS_API_NAME,
-                                                   _SERVERLESS_API_VERSION)
-    scheme, netloc, path, params, query, fragment = urlparse.urlparse(
-        global_endpoint)
-    netloc = '{}-{}'.format(self.region, netloc)
-    self.endpoint = urlparse.urlunparse(
-        (scheme, netloc, path, params, query, fragment))
+    global_endpoint = apis.GetEffectiveApiEndpoint(
+        global_methods.SERVERLESS_API_NAME,
+        global_methods.SERVERLESS_API_VERSION)
+    self.endpoint = DeriveRegionalEndpoint(global_endpoint, self.region)
     with _OverrideEndpointOverrides(self.endpoint):
       yield self
 
+  @property
+  def supports_one_platform(self):
+    return True
+
 
 def GetConnectionContext(args):
   """Gets the regional or GKE connection context.
@@ -195,18 +293,28 @@ def GetConnectionContext(args):
     args: Namespace, the args namespace.
 
   Raises:
-    ConfigurationError if cluster is specified without a location.
+    ArgumentError if region or cluster is not specified.
 
   Returns:
     A GKE or regional ConnectionInfo object.
   """
+  if flags.IsKubernetes(args):
+    config = flags.GetKubeconfig(args)
+    return _KubeconfigConnectionContext(config, args.context)
 
-  cluster_ref = args.CONCEPTS.cluster.Parse()
-  if cluster_ref:
+  if flags.IsGKE(args):
+    cluster_ref = args.CONCEPTS.cluster.Parse()
+    if not cluster_ref:
+      raise flags.ArgumentError(
+          'You must specify a cluster in a given location. '
+          'Either use the `--cluster` and `--cluster-location` flags '
+          'or set the run/cluster and run/cluster_location properties.')
     return _GKEConnectionContext(cluster_ref)
 
-  flags.ValidateClusterArgs(args)
-  region = flags.GetRegion(args, prompt=True)
-  if not region:
-    raise flags.ArgumentError('You must specify either a cluster or a region.')
-  return _RegionalConnectionContext(region)
+  if flags.IsManaged(args):
+    region = flags.GetRegion(args, prompt=True)
+    if not region:
+      raise flags.ArgumentError(
+          'You must specify a region. Either use the `--region` flag '
+          'or set the run/region property.')
+    return _RegionalConnectionContext(region)
