@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2019 Google Inc. All Rights Reserved.
+# Copyright 2019 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,24 +22,22 @@ from __future__ import unicode_literals
 import contextlib
 import copy
 import functools
-import glob
-import os
 import random
 import string
 from apitools.base.py import exceptions as api_exceptions
-from googlecloudsdk.api_lib.run import build_template
+from googlecloudsdk.api_lib.run import condition as run_condition
 from googlecloudsdk.api_lib.run import configuration
 from googlecloudsdk.api_lib.run import domain_mapping
-from googlecloudsdk.api_lib.run import k8s_object
+from googlecloudsdk.api_lib.run import global_methods
 from googlecloudsdk.api_lib.run import metric_names
 from googlecloudsdk.api_lib.run import revision
 from googlecloudsdk.api_lib.run import route
 from googlecloudsdk.api_lib.run import service
+from googlecloudsdk.api_lib.util import apis
 from googlecloudsdk.api_lib.util import apis_internal
 from googlecloudsdk.api_lib.util import exceptions as exceptions_util
 from googlecloudsdk.api_lib.util import waiter
 from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
-from googlecloudsdk.command_lib.run import deployable as deployable_pkg
 from googlecloudsdk.command_lib.run import exceptions as serverless_exceptions
 from googlecloudsdk.command_lib.run import resource_name_conversion
 from googlecloudsdk.command_lib.run import stages
@@ -62,6 +60,9 @@ NONCE_LABEL = 'client.knative.dev/nonce'
 # making it more likely to get a useful error message from the server.
 MAX_WAIT_MS = 660000
 
+ALLOW_UNAUTH_POLICY_BINDING_MEMBERS = ['allUsers']
+ALLOW_UNAUTH_POLICY_BINDING_ROLE = 'roles/run.invoker'
+
 
 class UnknownAPIError(exceptions.Error):
   pass
@@ -97,7 +98,12 @@ def Connect(conn_context):
   with conn_context as conn_info:
     # pylint: disable=protected-access
     client = apis_internal._GetClientInstance(
-        conn_info.api_name, conn_info.api_version, ca_certs=conn_info.ca_certs)
+        conn_info.api_name,
+        conn_info.api_version,
+        # Only check response if not connecting to GKE
+        check_response_func=apis.CheckResponseForApiEnablement
+        if conn_context.supports_one_platform else None,
+        ca_certs=conn_info.ca_certs)
     # pylint: enable=protected-access
     yield ServerlessOperations(
         client,
@@ -107,6 +113,30 @@ def Connect(conn_context):
         op_client)
 
 
+class DomainMappingResourceRecordPoller(waiter.OperationPoller):
+  """Poll for when a DomainMapping first has resourceRecords."""
+
+  def __init__(self, ops):
+    self._ops = ops
+
+  def IsDone(self, mapping):
+    if getattr(mapping.status, 'resourceRecords', None):
+      return True
+    conditions = mapping.conditions
+    # pylint: disable=g-bool-id-comparison
+    # False (indicating failure) as distinct from None (indicating not sure yet)
+    if conditions and conditions['Ready']['status'] is False:
+      return True
+    # pylint: enable=g-bool-id-comparison
+    return False
+
+  def GetResult(self, mapping):
+    return mapping
+
+  def Poll(self, domain_mapping_ref):
+    return self._ops.GetDomainMapping(domain_mapping_ref)
+
+
 class ConditionPoller(waiter.OperationPoller):
   """A poller for CloudRun resource creation or update.
 
@@ -140,13 +170,13 @@ class ConditionPoller(waiter.OperationPoller):
     """
     # _dependencies is a map of condition -> {preceding conditions}
     # It is meant to be checked off as we finish things.
-    self._dependencies = (
-        copy.deepcopy(dependencies) if dependencies is not None
-        else {k: set() for k in tracker})
-    for k in self._dependencies:
-      if k not in tracker:
-        raise ValueError(
-            'Condition {} is not in the dependency tracker.'.format(k))
+    self._dependencies = {k: set() for k in tracker}
+    if dependencies is not None:
+      for k in dependencies:
+        if k not in tracker:
+          raise ValueError(
+              'Condition {} is not in the dependency tracker.'.format(k))
+        self._dependencies[k] = copy.copy(dependencies[k])
     self._resource_getter = resource_getter
     self._tracker = tracker
     self._resource_fail_type = exceptions.Error
@@ -168,6 +198,26 @@ class ConditionPoller(waiter.OperationPoller):
       return False
     return conditions.IsTerminal()
 
+  def _PollTerminalSubconditions(self, conditions, ready_message):
+    for condition in conditions.TerminalSubconditions():
+      if condition not in self._dependencies:
+        continue
+      message = conditions[condition]['message']
+      status = conditions[condition]['status']
+      self._PossiblyUpdateMessage(condition, message, ready_message)
+      if status is None:
+        continue
+      elif status:
+        if self._PossiblyCompleteStage(
+            condition, message, conditions.IsReady()):
+          # Check all terminal subconditions again to ensure any stages that
+          # were unblocked by this stage completing are re-checked before we
+          # check the ready condition
+          self._PollTerminalSubconditions(conditions, ready_message)
+          break
+      else:
+        self._PossiblyFailStage(condition, message)
+
   def Poll(self, unused_ref):
     """Overrides.
 
@@ -187,18 +237,7 @@ class ConditionPoller(waiter.OperationPoller):
     if ready_message:
       self._tracker.UpdateHeaderMessage(ready_message)
 
-    for condition in conditions.TerminalSubconditions():
-      if condition not in self._dependencies:
-        continue
-      message = conditions[condition]['message']
-      status = conditions[condition]['status']
-      self._PossiblyUpdateMessage(condition, message, ready_message)
-      if status is None:
-        continue
-      elif status:
-        self._PossiblyCompleteStage(condition, message, conditions.IsReady())
-      else:
-        self._PossiblyFailStage(condition, message)
+    self._PollTerminalSubconditions(conditions, ready_message)
 
     if conditions.IsReady():
       self._tracker.UpdateHeaderMessage('Done.')
@@ -245,19 +284,23 @@ class ConditionPoller(waiter.OperationPoller):
       condition: str, The name of the condition whose stage should be completed.
       message: str, The detailed message for the condition.
       ready: boolean, True if the Ready condition is true.
+
+    Returns:
+      bool: True if stage was completed, False if no action taken
     """
     if self._tracker.IsComplete(condition):
-      return
+      return False
     # A blocked condition is likely to remain True (indicating the previous
     # operation concerning it was successful) until the blocking condition(s)
     # finish and it's time to switch to Unknown (the current operation
     # concerning it is in progress). Don't mark those done before they switch to
     # Unknown.
     if not self._tracker.IsRunning(condition):
-      return
+      return False
     self._RecordConditionComplete(condition)
     self._StartUnblocked()
     self._tracker.CompleteStage(condition, message)
+    return True
 
   def _StartUnblocked(self):
 
@@ -325,14 +368,6 @@ class ServiceConditionPoller(ConditionPoller):
     self._resource_fail_type = serverless_exceptions.DeploymentFailedError
 
 
-class DomainMappingConditionPoller(ConditionPoller):
-
-  def __init__(self, getter, tracker):
-    super(DomainMappingConditionPoller, self).__init__(
-        getter, tracker, stages.DomainMappingDependencies())
-    self._resource_fail_type = serverless_exceptions.DomainMappingCreationError
-
-
 def _Nonce():
   """Return a random string with unlikely collision to use as a nonce."""
   return ''.join(
@@ -345,9 +380,8 @@ class _NewRevisionForcingChange(config_changes_mod.ConfigChanger):
   def __init__(self, nonce):
     self._nonce = nonce
 
-  def AdjustConfiguration(self, config, metadata):
-    del metadata
-    config.revision_labels[NONCE_LABEL] = self._nonce
+  def Adjust(self, resource):
+    resource.template.labels[NONCE_LABEL] = self._nonce
 
 
 def _IsDigest(url):
@@ -380,17 +414,16 @@ class _SwitchToDigestChange(config_changes_mod.ConfigChanger):
   def __init__(self, base_revision):
     self._base_revision = base_revision
 
-  def AdjustConfiguration(self, config, metadata):
+  def Adjust(self, resource):
     if _IsDigest(self._base_revision.image):
       return
     if not self._base_revision.image_digest:
       return
 
-    annotations = k8s_object.AnnotationsFromMetadata(
-        config.MessagesModule(), metadata)
     # Mutates through to metadata: Save the by-tag user intent.
-    annotations[configuration.USER_IMAGE_ANNOTATION] = self._base_revision.image
-    config.image = self._base_revision.image_digest
+    resource.annotations[configuration.USER_IMAGE_ANNOTATION] = (
+        self._base_revision.image)
+    resource.template.image = self._base_revision.image_digest
 
 
 class ServerlessOperations(object):
@@ -412,7 +445,6 @@ class ServerlessOperations(object):
     self._client = client
     self._registry = resources.REGISTRY.Clone()
     self._registry.RegisterApiByName(api_name, api_version)
-    self._temporary_build_template_registry = {}
     self._op_client = op_client
     self._region = region
 
@@ -420,77 +452,6 @@ class ServerlessOperations(object):
   def _messages_module(self):
     return self._client.MESSAGES_MODULE
 
-  def IsSourceBranch(self):
-    # TODO(b/112662240): Remove once the build field is public
-    return hasattr(self._client.MESSAGES_MODULE.ConfigurationSpec, 'build')
-
-  # For internal-only source testing. Codepaths inaccessable except on
-  # build from dev branch.
-  # TODO(b/112662240): productionalize when source is landing
-  def _TemporaryBuildTemplateRegistry(self, namespace_ref):
-    """Return the list of build templates available, mocking the server."""
-    if namespace_ref.RelativeName() in self._temporary_build_template_registry:
-      return self._temporary_build_template_registry[
-          namespace_ref.RelativeName()]
-
-    detect = build_template.BuildTemplate.New(
-        self._client, 'default')
-    detect.name = 'detect'
-    detect.annotations[build_template.IGNORE_GLOB_ANNOTATION] = (
-        '["/*", "!package.json","!Pipfile.lock"]')
-
-    nodejs_8_9_4 = build_template.BuildTemplate.New(
-        self._client, 'default')
-    nodejs_8_9_4.name = 'nodejs_8_9_4'
-    nodejs_8_9_4.annotations[build_template.IGNORE_GLOB_ANNOTATION] = (
-        '["node_modules/"]')
-    nodejs_8_9_4.labels[build_template.LANGUAGE_LABEL] = 'nodejs'
-    nodejs_8_9_4.labels[build_template.VERSION_LABEL] = '8.9.4'
-    nodejs_8_9_4.annotations[build_template.DEV_IMAGE_ANNOTATION] = (
-        'gcr.io/local-run-demo/nodejs_dev:latest')
-
-    go_1_10_1 = build_template.BuildTemplate.New(
-        self._client, 'default')
-    go_1_10_1.name = 'go_1_10_1'
-    go_1_10_1.labels[build_template.LANGUAGE_LABEL] = 'go'
-    go_1_10_1.labels[build_template.VERSION_LABEL] = '1.10.1'
-    lst = [detect, nodejs_8_9_4, go_1_10_1]
-    self._temporary_build_template_registry[namespace_ref.RelativeName()] = lst
-    return lst
-
-  def Detect(self, namespace_ref, source_ref, function_entrypoint=None):
-    """Detects important properties and returns a Deployable.
-
-    Args:
-      namespace_ref: str, the namespace to look for build templates in
-      source_ref: source_ref.SourceRef, refers to some source code
-      function_entrypoint: str, allows you to specify this is a function, and
-                           the function to run.
-
-    Returns:
-      a new Deployable referring to the source
-    """
-    template = self._DetectBuildTemplate(namespace_ref, source_ref)
-
-    if (source_ref.source_type == source_ref.SourceType.IMAGE
-        and not template and not function_entrypoint):
-      return deployable_pkg.ServerlessContainer(source_ref)
-
-    if not self.IsSourceBranch():
-      raise serverless_exceptions.UnknownDeployableError()
-    # TODO(b/112662240): Put at top when source lands.
-    from googlecloudsdk.command_lib.run import source_deployable  # pylint: disable=g-import-not-at-top
-    if (function_entrypoint and
-        template and
-        source_ref.source_type == source_ref.SourceType.DIRECTORY):
-      return source_deployable.ServerlessFunction(source_ref, template,
-                                                  function_entrypoint)
-
-    if (source_ref.source_type == source_ref.SourceType.DIRECTORY and
-        template and
-        not function_entrypoint):
-      return source_deployable.ServerlessApp(source_ref, template)
-
   def GetRevision(self, revision_ref):
     """Get the revision.
 
@@ -537,24 +498,6 @@ class ServerlessOperations(object):
     except api_exceptions.HttpNotFoundError:
       return None
 
-  def _GetBuildTemplateByName(self, namespace_ref, name):
-    """Return the BuildTemplate with the given name, or None."""
-    # Implementation to be replaced once the concept exists on the server.
-    for templ in self._TemporaryBuildTemplateRegistry(namespace_ref):
-      if templ.name == name:
-        return templ
-    return None
-
-  def _GetBuildTemplateByLanguageVersion(self, namespace_ref,
-                                         language, version):
-    """Return the BuildTemplate with the given language & version, or None."""
-    # Implementation to be replaced once the concept exists on the server.
-    del namespace_ref
-    for templ in self._temporary_build_template_registry:
-      if (templ.language, templ.version) == (language, version):
-        return templ
-    return None
-
   def WaitForCondition(self, poller):
     """Wait for a configuration to be ready in latest revision.
 
@@ -613,23 +556,6 @@ class ServerlessOperations(object):
 
     return serv_route.active_revisions
 
-  def _DetectBuildTemplate(self, namespace_ref, source_ref):
-    """Determine the appropriate build template from source.
-
-    Args:
-      namespace_ref: Resource, namespace to find build templates in.
-      source_ref: SourceRef, The service's image repo or source directory.
-
-    Returns:
-      The detected build template name.
-    """
-    if source_ref.source_type == source_ref.SourceType.IMAGE:
-      return None
-    elif glob.glob(os.path.join(source_ref.source_path, '*.go')):
-      return self._GetBuildTemplateByName(namespace_ref, 'go_1_10_1')
-    else:
-      return self._GetBuildTemplateByName(namespace_ref, 'nodejs_8_9_4')
-
   def ListServices(self, namespace_ref):
     messages = self._messages_module
     request = messages.RunNamespacesServicesListRequest(
@@ -768,7 +694,7 @@ class ServerlessOperations(object):
     response = self._client.namespaces_revisions.List(request)
     return [revision.Revision(item, messages) for item in response.items]
 
-  def _GetBaseRevision(self, config, metadata, status):
+  def _GetBaseRevision(self, template, metadata, status):
     """Return a Revision for use as the "base revision" for a change.
 
     When making a change that should not affect the code running, the
@@ -781,7 +707,7 @@ class ServerlessOperations(object):
         the latestCreatedRevision in status.
 
     Arguments:
-      config: Configuration, the configuration to get the base revision of.
+      template: Revision, the revision template to get the base revision of.
         May have been derived from a Service.
       metadata: ObjectMeta, the metadata from the top-level object
       status: Union[ConfigurationStatus, ServiceStatus], the status of the top-
@@ -792,7 +718,7 @@ class ServerlessOperations(object):
     """
     # Or returns None if not available by nonce & the control plane has not
     # implemented latestCreatedRevisionName on the Service object yet.
-    base_revision_nonce = config.revision_labels.get(NONCE_LABEL, None)
+    base_revision_nonce = template.labels.get(NONCE_LABEL, None)
     base_revision = None
     if base_revision_nonce:
       try:
@@ -820,9 +746,9 @@ class ServerlessOperations(object):
 
   def _EnsureImageDigest(self, serv, config_changes):
     """Make config_changes include switch by-digest image if not so already."""
-    if not _IsDigest(serv.configuration.image):
+    if not _IsDigest(serv.template.image):
       base_revision = self._GetBaseRevision(
-          serv.configuration, serv.metadata, serv.status)
+          serv.template, serv.metadata, serv.status)
       if base_revision:
         config_changes.append(_SwitchToDigestChange(base_revision))
 
@@ -859,12 +785,12 @@ class ServerlessOperations(object):
           pass
         elif private_endpoint:
           serv.labels[service.ENDPOINT_VISIBILITY] = service.CLUSTER_LOCAL
-        else:
+        elif service.ENDPOINT_VISIBILITY in serv.labels:
           del serv.labels[service.ENDPOINT_VISIBILITY]
 
         # PUT the changed Service
         for config_change in config_changes:
-          config_change.AdjustConfiguration(serv.configuration, serv.metadata)
+          config_change.Adjust(serv)
         serv_name = service_ref.RelativeName()
         serv_update_req = (
             messages.RunNamespacesServicesReplaceServiceRequest(
@@ -885,8 +811,7 @@ class ServerlessOperations(object):
         new_serv.name = service_ref.servicesId
         parent = service_ref.Parent().RelativeName()
         for config_change in config_changes:
-          config_change.AdjustConfiguration(new_serv.configuration,
-                                            new_serv.metadata)
+          config_change.Adjust(new_serv)
         serv_create_req = (
             messages.RunNamespacesServicesCreateRequest(
                 service=new_serv.Message(),
@@ -903,16 +828,29 @@ class ServerlessOperations(object):
           exceptions.reraise(serverless_exceptions.BadImageError(e))
       exceptions.reraise(e)
     except api_exceptions.HttpNotFoundError as e:
-      # TODO(b/118339293): List available regions to check whether provided
-      # region is invalid or not.
-      raise serverless_exceptions.DeploymentFailedError(
-          'Deployment endpoint was not found. Perhaps the provided '
-          'region was invalid. Set the `run/region` property to a valid '
-          'region and retry. Ex: `gcloud config set run/region us-central1`')
+      error_msg = 'Deployment endpoint was not found.'
+      if not self._region:
+        all_clusters = global_methods.ListClusters()
+        clusters = ['* {} in {}'.format(c.name, c.zone) for c in all_clusters]
+        error_msg += (' Perhaps the provided cluster was invalid or '
+                      'does not have Cloud Run enabled. Pass the '
+                      '`--cluster` and `--cluster-location` flags or set the '
+                      '`run/cluster` and `run/cluster_location` properties to '
+                      'a valid cluster and zone and retry.'
+                      '\nAvailable clusters:\n{}'.format('\n'.join(clusters)))
+      else:
+        all_regions = global_methods.ListRegions(self._op_client)
+        if self._region not in all_regions:
+          regions = ['* {}'.format(r) for r in all_regions]
+          error_msg += (' The provided region was invalid. '
+                        'Pass the `--region` flag or set the '
+                        '`run/region` property to a valid region and retry.'
+                        '\nAvailable regions:\n{}'.format('\n'.join(regions)))
+      raise serverless_exceptions.DeploymentFailedError(error_msg)
 
   def ReleaseService(self, service_ref, config_changes, tracker=None,
                      asyn=False, private_endpoint=None,
-                     allow_unauthenticated=False):
+                     allow_unauthenticated=None):
     """Change the given service in prod using the given config_changes.
 
     Ensures a new revision is always created, even if the spec of the revision
@@ -920,7 +858,7 @@ class ServerlessOperations(object):
 
     Arguments:
       service_ref: Resource, the service to release
-      config_changes: list, objects that implement AdjustConfiguration().
+      config_changes: list, objects that implement Adjust().
       tracker: StagedProgressTracker, to report on the progress of releasing.
       asyn: bool, if True, release asyncronously
       private_endpoint: bool, True if creating a new Service for
@@ -929,24 +867,49 @@ class ServerlessOperations(object):
         its existing visibility should remain unchanged.
       allow_unauthenticated: bool, True if creating a hosted Cloud Run
         service which should also have its IAM policy set to allow
-        unauthenticated access.
+        unauthenticated access. False if removing the IAM policy to allow
+        unauthenticated access from a service.
     """
     if tracker is None:
       tracker = progress_tracker.NoOpStagedProgressTracker(
-          stages.ServiceStages(), interruptable=True, aborted_message='aborted')
-    with_code = any(
-        isinstance(c, deployable_pkg.Deployable) for c in config_changes)
+          stages.ServiceStages(allow_unauthenticated is not None),
+          interruptable=True, aborted_message='aborted')
+    with_image = any(
+        isinstance(c, config_changes_mod.ImageChange) for c in config_changes)
     self._UpdateOrCreateService(
-        service_ref, config_changes, with_code, private_endpoint)
-    if allow_unauthenticated:
-      self.AddIamPolicyBinding(service_ref, ['allUsers'], 'roles/run.invoker')
+        service_ref, config_changes, with_image, private_endpoint)
+
+    if allow_unauthenticated is not None:
+      try:
+        tracker.StartStage(stages.SERVICE_IAM_POLICY_SET)
+        tracker.UpdateStage(stages.SERVICE_IAM_POLICY_SET, '')
+        self.AddOrRemoveIamPolicyBinding(service_ref, allow_unauthenticated,
+                                         ALLOW_UNAUTH_POLICY_BINDING_MEMBERS,
+                                         ALLOW_UNAUTH_POLICY_BINDING_ROLE)
+        tracker.CompleteStage(stages.SERVICE_IAM_POLICY_SET)
+      except api_exceptions.HttpError:
+        warning_message = (
+            'Setting IAM policy failed, try "gcloud beta run services '
+            '{}-iam-policy-binding --region={region} --member=allUsers '
+            '--role=roles/run.invoker {service}"'.format(
+                'add' if allow_unauthenticated else 'remove',
+                region=self._region,
+                service=service_ref.servicesId))
+        tracker.CompleteStageWithWarning(
+            stages.SERVICE_IAM_POLICY_SET, warning_message=warning_message)
+
     if not asyn:
       getter = functools.partial(self.GetService, service_ref)
-      self.WaitForCondition(ServiceConditionPoller(getter, tracker))
+      poller = ServiceConditionPoller(getter, tracker)
+      self.WaitForCondition(poller)
+      for msg in run_condition.GetNonTerminalMessages(poller.GetConditions()):
+        tracker.AddWarning(msg)
 
   def ListRevisions(self, namespace_ref, service_name):
     """List all revisions for the given service.
 
+    Revision list gets sorted by service name and creation timestamp.
+
     Args:
       namespace_ref: Resource, namespace to list revisions in
       service_name: str, The service for which to list revisions.
@@ -965,7 +928,13 @@ class ServerlessOperations(object):
           service_name)
     with metrics.RecordDuration(metric_names.LIST_REVISIONS):
       response = self._client.namespaces_revisions.List(request)
-    return [revision.Revision(item, messages) for item in response.items]
+
+    # Server does not sort the response so we'll need to sort client-side
+    revisions = [revision.Revision(item, messages) for item in response.items]
+    # Newest first
+    revisions.sort(key=lambda r: r.creation_timestamp, reverse=True)
+    revisions.sort(key=lambda r: r.service_name)
+    return revisions
 
   def ListDomainMappings(self, namespace_ref):
     """List all domain mappings.
@@ -994,6 +963,7 @@ class ServerlessOperations(object):
     Returns:
       A domain_mapping.DomainMapping object.
     """
+
     messages = self._messages_module
     new_mapping = domain_mapping.DomainMapping.New(
         self._client, domain_mapping_ref.namespacesId)
@@ -1005,14 +975,22 @@ class ServerlessOperations(object):
         parent=domain_mapping_ref.Parent().RelativeName())
     with metrics.RecordDuration(metric_names.CREATE_DOMAIN_MAPPING):
       response = self._client.namespaces_domainmappings.Create(request)
+      # 'run domain-mappings create' is synchronous. Poll for its completion.
+      with progress_tracker.ProgressTracker('Creating...'):
+        mapping = waiter.PollUntilDone(
+            DomainMappingResourceRecordPoller(self), domain_mapping_ref)
+      ready = mapping.conditions.get('Ready')
+      records = getattr(mapping.status, 'resourceRecords', None)
+      message = None
+      if ready and ready.get('message'):
+        message = ready['message']
+      if not records:
+        raise serverless_exceptions.DomainMappingCreationError(
+            message or 'Could not create domain mapping.')
+      if message:
+        log.status.Print(message)
+      return records
 
-    # 'run domain-mappings create' is synchronous. Poll for its completion.
-    getter = functools.partial(self.GetDomainMapping, domain_mapping_ref)
-    with progress_tracker.StagedProgressTracker(
-        'Creating...',
-        stages.DomainMappingStages(),
-        failure_message='Domain mapping failed') as tracker:
-      self.WaitForCondition(DomainMappingConditionPoller(getter, tracker))
     return domain_mapping.DomainMapping(response, messages)
 
   def DeleteDomainMapping(self, domain_mapping_ref):
@@ -1052,8 +1030,9 @@ class ServerlessOperations(object):
     response = self._op_client.projects_locations_services.GetIamPolicy(request)
     return response
 
-  def AddIamPolicyBinding(self, service_ref, members=None, role=None):
-    """Add the given IAM policy binding to the provided service.
+  def AddOrRemoveIamPolicyBinding(self, service_ref, add_binding=True,
+                                  members=None, role=None):
+    """Add or remove the given IAM policy binding to the provided service.
 
     If no members or role are provided, set the IAM policy to the current IAM
     policy. This is useful for checking whether the authenticated user has
@@ -1061,29 +1040,34 @@ class ServerlessOperations(object):
 
     Args:
       service_ref: str, The service to which to add the IAM policy.
+      add_binding: bool, Whether to add to or remove from the IAM policy.
       members: [str], The users for which the binding applies.
       role: str, The role to grant the provided members.
 
     Returns:
-      The IAM policy with the newly added binding.
+      A google.iam.v1.TestIamPermissionsResponse.
     """
     messages = self._messages_module
     oneplatform_service = resource_name_conversion.K8sToOnePlatform(
         service_ref, self._region)
     policy = self._GetIamPolicy(oneplatform_service)
+    # Don't modify bindings if not members or roles provided
     if members and role:
-      policy.bindings.append(
-          messages.Binding(members=members, role=role))
+      binding = messages.Binding(members=members, role=role)
+      if add_binding:
+        policy.bindings.append(binding)
+      else:
+        # Remove bindings that exactly match the provided members and role
+        policy.bindings = [b for b in policy.bindings if b != binding]
     request = messages.RunProjectsLocationsServicesSetIamPolicyRequest(
         resource=str(oneplatform_service),
         setIamPolicyRequest=messages.SetIamPolicyRequest(policy=policy))
-    return self._op_client.projects_locations_services.SetIamPolicy(request)
+    result = self._op_client.projects_locations_services.SetIamPolicy(request)
+    return result
 
-  def CanAddIamPolicyBinding(self, service_ref):
+  def CanSetIamPolicyBinding(self, service_ref):
     try:
-      self.AddIamPolicyBinding(service_ref)
+      self.AddOrRemoveIamPolicyBinding(service_ref)
       return True
     except api_exceptions.HttpError:
       return False
-
-
