@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -159,6 +159,14 @@ def AddPassword(parser):
       help='Cloud SQL user\'s password.')
 
 
+def AddRootPassword(parser):
+  """Add the root password field to the parser."""
+  parser.add_argument(
+      '--root-password',
+      required=False,
+      help='Root Cloud SQL user\'s password.')
+
+
 def AddPromptForPassword(parser):
   parser.add_argument(
       '--prompt-for-password',
@@ -189,7 +197,10 @@ def AddAssignIp(parser, show_negated_in_help=False):
   kwargs = _GetKwargsForBoolFlag(show_negated_in_help)
   parser.add_argument(
       '--assign-ip',
-      help='If provided, instance must be assigned an IP address.', **kwargs)
+      help='Assign an IPv4 external address to this instance.  This setting is '
+           'enabled by default.  To create an instance which only has a '
+           'private IP, use --no-assign-ip and specify a private network.',
+      **kwargs)
 
 
 def AddAuthorizedGAEApps(parser, update=False):
@@ -256,6 +267,26 @@ def AddDatabaseFlags(parser, update=False):
       help=help_)
 
 
+def AddDatabaseVersion(parser, restrict_choices=True):
+  """Adds `--database-version` to the parser with choices restricted or not."""
+  choices = ['MYSQL_5_5', 'MYSQL_5_6', 'MYSQL_5_7', 'POSTGRES_9_6']
+  help_text = (
+      'The database engine type and version. If left unspecified, the API '
+      'defaults will be used.')
+  if not restrict_choices:
+    help_text = ' '.join([
+        help_text,
+        'See the list of database versions at '
+        'https://cloud.google.com/sql/docs/mysql/admin-api/v1beta4/instances'
+        '#databaseVersion'
+    ])
+  parser.add_argument(
+      '--database-version',
+      required=False,
+      choices=choices if restrict_choices else None,
+      help=help_text)
+
+
 def AddCPU(parser):
   parser.add_argument(
       '--cpu',
@@ -467,11 +498,12 @@ def AddStorageSize(parser):
       '--storage-size',
       type=arg_parsers.BinarySize(
           lower_bound='10GB',
-          upper_bound='10230GB',
+          upper_bound='30710GB',
           suggested_binary_size_scales=['GB']),
       help=('Amount of storage allocated to the instance. Must be an integer '
-            'number of GB between 10GB and 10230GB inclusive. The default is '
-            '10GB.'))
+            'number of GB. The default is 10GB. Information on storage '
+            'limits can be found here: '
+            'https://cloud.google.com/sql/docs/quotas#storage_limits'))
 
 
 def AddTier(parser, is_patch=False):
