@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,6 +31,7 @@ from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import execution_utils
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
+from googlecloudsdk.core.console import console_io
 
 DEFAULT_RELEASE_TRACK = base.ReleaseTrack.GA
 
@@ -70,6 +71,27 @@ def ConnectToInstance(cmd_args, sql_user):
     log.Print(info_holder.InfoHolder())
 
 
+def _GetAndValidateCmekKeyName(args):
+  """Parses the CMEK resource arg, makes sure the key format was correct."""
+  kms_ref = args.CONCEPTS.kms_key.Parse()
+  if kms_ref:
+    _ShowCmekPrompt()
+    return kms_ref.RelativeName()
+  else:
+    # Check for partially specified disk-encryption-key.
+    for keyword in [
+        'disk-encryption-key', 'disk-encryption-key-keyring',
+        'disk-encryption-key-location', 'disk-encryption-key-project'
+    ]:
+      if getattr(args, keyword.replace('-', '_'), None):
+        raise exceptions.InvalidArgumentException('--disk-encryption-key',
+                                                  'not fully specified.')
+
+
+def _GetZone(args):
+  return args.zone or args.gce_zone
+
+
 def _IsAlpha(release_track):
   return release_track == base.ReleaseTrack.ALPHA
 
@@ -82,6 +104,18 @@ def _ParseActivationPolicy(policy):
   return policy.replace('-', '_').upper() if policy else None
 
 
+# TODO(b/122660263): Remove when V1 instances are no longer supported.
+def ShowV1DeprecationWarning(plural=False):
+  message = (
+      'Upgrade your First Generation instance{} to Second Generation before we '
+      'auto-upgrade {} on March 4, 2020, ahead of the full decommission of '
+      'First Generation on March 25, 2020.')
+  if plural:
+    log.warning(message.format('s', 'them'))
+  else:
+    log.warning(message.format('', 'it'))
+
+
 def ShowZoneDeprecationWarnings(args):
   """Show warnings if both region and zone are specified or neither is.
 
@@ -91,16 +125,30 @@ def ShowZoneDeprecationWarnings(args):
   """
 
   region_specified = args.IsSpecified('region')
-  zone_specified = args.IsSpecified('gce_zone')
+  zone_specified = args.IsSpecified('gce_zone') or args.IsSpecified('zone')
 
+  # TODO(b/73362371): Remove this check; user must specify a location flag.
   if not (region_specified or zone_specified):
-    log.warning('Starting with release 218.0.0, you will need to specify '
+    log.warning('Starting with release 233.0.0, you will need to specify '
                 'either a region or a zone to create an instance.')
 
-  # TODO(b/73362466): Remove this check; these flags will be mutually exclusive.
-  if region_specified and zone_specified:
-    log.warning('Zone will override region. Starting with release 204.0.0, '
-                'region and zone will become mutually exclusive arguments.')
+
+def ShowCmekWarning(resource_type_label, instance_type_label):
+  log.warning(
+      'Your {} will be encrypted with {}\'s customer-managed encryption key. '
+      'If anyone destroys this key, all data encrypted with it will be '
+      'permanently lost.'.format(resource_type_label, instance_type_label))
+
+
+def _ShowCmekPrompt():
+  log.warning(
+      'You are creating a Cloud SQL instance encrypted with a customer-managed '
+      'key. If anyone destroys a customer-managed key, all data encrypted with '
+      'it will be permanently lost.\n\n'
+      'Make sure you have the following permission, which is needed for this '
+      'instance\'s service account to encrypt/decrypt with the selected key: '
+      'resourcemanager.projects.setIamPolicy\n')
+  console_io.PromptContinue(cancel_on_no=True)
 
 
 class _BaseInstances(object):
@@ -154,9 +202,9 @@ class _BaseInstances(object):
       if args.require_ssl is not None:
         settings.ipConfiguration.requireSsl = args.require_ssl
 
-    if any([args.follow_gae_app, args.gce_zone]):
+    if any([args.follow_gae_app, _GetZone(args)]):
       settings.locationPreference = sql_messages.LocationPreference(
-          followGaeApplication=args.follow_gae_app, zone=args.gce_zone)
+          followGaeApplication=args.follow_gae_app, zone=_GetZone(args))
 
     if args.storage_size:
       settings.dataDiskSizeGb = int(args.storage_size / constants.BYTES_TO_GB)
@@ -254,9 +302,9 @@ class _BaseInstances(object):
     if args.clear_gae_apps:
       settings.authorizedGaeApplications = []
 
-    if any([args.follow_gae_app, args.gce_zone]):
+    if any([args.follow_gae_app, _GetZone(args)]):
       settings.locationPreference = sql_messages.LocationPreference(
-          followGaeApplication=args.follow_gae_app, zone=args.gce_zone)
+          followGaeApplication=args.follow_gae_app, zone=_GetZone(args))
 
     if args.clear_authorized_networks:
       if not settings.ipConfiguration:
@@ -345,9 +393,10 @@ class _BaseInstances(object):
     instance_resource = cls._ConstructBaseInstanceFromArgs(
         sql_messages, args, original, instance_ref)
 
-    instance_resource.region = reducers.Region(args.region, args.gce_zone)
+    instance_resource.region = reducers.Region(args.region, _GetZone(args))
     instance_resource.databaseVersion = args.database_version
     instance_resource.masterInstanceName = args.master_instance_name
+    instance_resource.rootPassword = args.root_password
 
     # BETA: Set the host port and return early if external master instance.
     if _IsBetaOrNewer(release_track) and args.IsSpecified('source_ip_address'):
@@ -400,6 +449,13 @@ class _BaseInstances(object):
           args.master_dump_file_path, args.master_ca_certificate_path,
           args.client_certificate_path, args.client_key_path)
 
+    # ALPHA args.
+    if _IsAlpha(release_track):
+      key_name = _GetAndValidateCmekKeyName(args)
+      if key_name:
+        config = sql_messages.DiskEncryptionConfiguration(kmsKeyName=key_name)
+        instance_resource.diskEncryptionConfiguration = config
+
     return instance_resource
 
   @classmethod
