@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -101,6 +101,7 @@ class Request(object):
 
   def __init__(self, command_type, data):
     self.collection = data['collection']
+    self.disable_resource_check = data.get('disable_resource_check')
     self.display_resource_type = data.get('display_resource_type')
     self.api_version = data.get('api_version')
     self.method = data.get('method', command_type.default_method)
@@ -115,6 +116,7 @@ class Request(object):
     self.modify_request_hooks = [
         util.Hook.FromPath(p) for p in data.get('modify_request_hooks', [])]
     self.create_request_hook = util.Hook.FromData(data, 'create_request_hook')
+    self.modify_method_hook = util.Hook.FromData(data, 'modify_method_hook')
     self.issue_request_hook = util.Hook.FromData(data, 'issue_request_hook')
     self.use_relative_name = data.get('use_relative_name', True)
 
@@ -158,6 +160,8 @@ class Async(object):
     self.result_attribute = data.get('result_attribute')
     self.state = AsyncStateField(data.get('state', {}))
     self.error = AsyncErrorField(data.get('error', {}))
+    self.modify_request_hooks = [
+        util.Hook.FromPath(p) for p in data.get('modify_request_hooks', [])]
 
 
 class IamData(object):
@@ -193,6 +197,14 @@ class Arguments(object):
         data, 'additional_arguments_hook')
     self.params = [
         Argument.FromData(param_data) for param_data in data.get('params', [])]
+    self.labels = Labels(data.get('labels')) if data.get('labels') else None
+
+
+class Labels(object):
+  """Everything about labels of GCP resources."""
+
+  def __init__(self, data):
+    self.api_field = data['api_field']
 
 
 class Argument(object):
@@ -422,6 +434,7 @@ class Output(object):
 
   def __init__(self, data):
     self.format = data.get('format')
+    self.flatten = data.get('flatten')
 
 
 class UpdateData(object):
