@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,8 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import json
+
 from apitools.base.protorpclite import messages as apitools_messages
 from apitools.base.py import encoding
 from apitools.base.py.exceptions import HttpBadRequestError
@@ -38,12 +40,15 @@ from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.apis import registry
 from googlecloudsdk.command_lib.util.apis import update
 from googlecloudsdk.command_lib.util.apis import yaml_command_schema
+from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import resources
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.resource import resource_transform
 
+import six
+
 
 class Translator(command_loading.YamlCommandTranslator):
   """Class that implements the calliope translator interface."""
@@ -281,6 +286,8 @@ class CommandBuilder(object):
         self._CommonArgs(parser)
         if self.spec.async:
           base.ASYNC_FLAG.AddToParser(parser)
+        if self.spec.arguments.labels:
+          labels_util.AddCreateLabelsFlags(parser)
 
       def Run(self_, args):
         ref, response = self._CommonRun(args)
@@ -302,7 +309,11 @@ class CommandBuilder(object):
           # Data on responses from operation polling is stored in
           # additionalProperties, so convert to dict for consistent behavior.
           response_obj = encoding.MessageToDict(response)
+          # If the response is an operation that has a 'response' property that
+          # has a name, use that. Otherwise, use the 'name' property.
           full_name = response_obj.get('response', {}).get('name')
+          if not full_name:
+            full_name = response_obj.get('name')
           resource_name = resource_transform.TransformBaseName(full_name)
         else:
           resource_name = self._GetDisplayName(ref, args)
@@ -624,6 +635,8 @@ class CommandBuilder(object):
         self._CommonArgs(parser)
         if self.spec.async:
           base.ASYNC_FLAG.AddToParser(parser)
+        if self.spec.arguments.labels:
+          labels_util.AddUpdateLabelsFlags(parser)
 
       def Run(self_, args):
         # Check if mask is required for an update request, if required, return
@@ -676,6 +689,8 @@ class CommandBuilder(object):
         arg.AddToParser(parser)
     if self.spec.output.format:
       parser.display_info.AddFormat(self.spec.output.format)
+    if self.spec.output.flatten:
+      parser.display_info.AddFlatten(self.spec.output.flatten)
 
   def _CommonRun(self, args, existing_message=None):
     """Performs run actions common to all commands.
@@ -699,6 +714,12 @@ class CommandBuilder(object):
                        self._GetDisplayName(ref, args)),
           throw_if_unattended=True, cancel_on_no=True)
 
+    if self.spec.request.modify_method_hook:
+      self.spec.request.method = self.spec.request.modify_method_hook(ref, args)
+      self.method = registry.GetMethod(
+          self.spec.request.collection, self.spec.request.method,
+          self.spec.request.api_version)
+
     if self.spec.request.issue_request_hook:
       # Making the request is overridden, just call into the custom code.
       return ref, self.spec.request.issue_request_hook(ref, args)
@@ -712,9 +733,12 @@ class CommandBuilder(object):
           args,
           self.spec.request.static_fields,
           self.spec.request.resource_method_params,
+          self.spec.arguments.labels,
+          self.spec.command_type,
           use_relative_name=self.spec.request.use_relative_name,
           parse_resource_into_request=parse_resource,
-          existing_message=existing_message)
+          existing_message=existing_message,
+          override_method=self.method)
       for hook in self.spec.request.modify_request_hooks:
         request = hook(ref, args, request)
 
@@ -853,7 +877,7 @@ class CommandBuilder(object):
   def _WaitForOperation(self, operation_ref, resource_ref,
                         extract_resource_result, args=None):
     poller = AsyncOperationPoller(
-        self.spec, resource_ref if extract_resource_result else None)
+        self.spec, resource_ref if extract_resource_result else None, args)
     progress_string = self._Format(
         'Waiting for operation [{{{}}}] to complete'.format(
             yaml_command_schema.NAME_FORMAT_KEY),
@@ -999,7 +1023,7 @@ class CommandBuilder(object):
 class AsyncOperationPoller(waiter.OperationPoller):
   """An implementation of a operation poller."""
 
-  def __init__(self, spec, resource_ref):
+  def __init__(self, spec, resource_ref, args):
     """Creates the poller.
 
     Args:
@@ -1009,6 +1033,7 @@ class AsyncOperationPoller(waiter.OperationPoller):
         being operated on (not the operation itself). If None, the operation
         will just be returned when it is done instead of getting the resulting
         resource.
+      args: Namespace, The args namespace.
     """
     self.spec = spec
     self.resource_ref = resource_ref
@@ -1017,6 +1042,7 @@ class AsyncOperationPoller(waiter.OperationPoller):
     self.method = registry.GetMethod(
         spec.async.collection, spec.async.method,
         api_version=spec.async.api_version or spec.request.api_version)
+    self.args = args
 
   def IsDone(self, operation):
     """Overrides."""
@@ -1031,7 +1057,7 @@ class AsyncOperationPoller(waiter.OperationPoller):
         error = 'The operation failed.'
       # If we succeeded but there is an error, or if an error was detected.
       if error:
-        raise waiter.OperationError(error)
+        raise waiter.OperationError(SerializeError(error))
       return True
 
     return False
@@ -1053,7 +1079,10 @@ class AsyncOperationPoller(waiter.OperationPoller):
             self.spec.async.operation_get_method_params.get(f.name, f.name),
             relative_name)
         for f in request_type.all_fields()}
-    return self.method.Call(request_type(**fields))
+    request = request_type(**fields)
+    for hook in self.spec.async.modify_request_hooks:
+      request = hook(operation_ref, self.args, request)
+    return self.method.Call(request)
 
   def GetResult(self, operation):
     """Overrides.
@@ -1078,6 +1107,21 @@ class AsyncOperationPoller(waiter.OperationPoller):
         api_version=self.spec.request.api_version)
 
 
+def SerializeError(error):
+  """Serializes the error message for better format."""
+  if isinstance(error, six.string_types):
+    return error
+  try:
+    return json.dumps(
+        encoding.MessageToDict(error),
+        indent=2,
+        sort_keys=True,
+        separators=(',', ': '))
+  except Exception:  # pylint: disable=broad-except
+    # try the best, fall back to return error
+    return error
+
+
 def _GetAttribute(obj, attr_path):
   """Gets attributes and sub-attributes out of an object.
 
