@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -148,7 +148,6 @@ class Environment(object):
     """
     self.suite = suite
     self.bin_path = bin_path
-    # So pytype is aware of attributes.
     self.ssh = None
     self.ssh_term = None
     self.scp = None
@@ -313,11 +312,10 @@ class Keys(object):
     private_key_file = os.path.realpath(files.ExpandHomeDir(key_file))
     self.dir = os.path.dirname(private_key_file)
     self.env = env or Environment.Current()
-    # TODO(b/71388306): Enums aren't handled well by pytype.
     self.keys = {
         _KeyFileKind.PRIVATE: self.KeyFileData(private_key_file),
         _KeyFileKind.PUBLIC: self.KeyFileData(private_key_file + '.pub')
-    }  # type: dict[enum.Enum, Keys.KeyFileData]
+    }
     if self.env.suite is Suite.PUTTY:
       self.keys[_KeyFileKind.PPK] = self.KeyFileData(private_key_file + '.ppk')
 
@@ -346,13 +344,13 @@ class Keys(object):
     status_padding = 0
     for kind in self.keys:
       data = self.keys[kind]
-      key_padding = max(key_padding, len(kind.value))  # pytype: disable=attribute-error
-      status_padding = max(status_padding, len(data.status.value))  # pytype: disable=attribute-error
+      key_padding = max(key_padding, len(kind.value))
+      status_padding = max(status_padding, len(data.status.value))
     for kind in self.keys:
       data = self.keys[kind]
       messages.append('{} {} [{}]\n'.format(
-          (kind.value + ' key').ljust(key_padding + 4),  # pytype: disable=attribute-error
-          ('(' + data.status.value + ')') .ljust(status_padding + 2),  # pytype: disable=attribute-error
+          (kind.value + ' key').ljust(key_padding + 4),
+          ('(' + data.status.value + ')') .ljust(status_padding + 2),
           data.filename))
     messages.sort()
     return ''.join(messages)
@@ -371,7 +369,7 @@ class Keys(object):
     """
     def ValidateFile(kind):
       status_or_line = self._WarnOrReadFirstKeyLine(self.keys[kind].filename,
-                                                    kind.value)  # pytype: disable=attribute-error
+                                                    kind.value)
       if isinstance(status_or_line, KeyFileStatus):
         return status_or_line
       else:  # returned line - present
@@ -472,7 +470,6 @@ class Keys(object):
     return status
 
   def GetPublicKey(self):
-    # type: () -> Keys.PublicKey
     """Returns the public key verbatim from file as a string.
 
     Precondition: The public key must exist. Run Keys.EnsureKeysExist() prior.
@@ -614,6 +611,42 @@ class KnownHosts(object):
     else:
       self.known_hosts.append(new_key_entry)
 
+  def AddMultiple(self, hostname, host_keys, overwrite=False):
+    """Add or update multiple entries for the given hostname.
+
+    If there is no entry for the given hostname, the keys will be added. If
+    there is an entry already, and overwrite keys is False, nothing will be
+    changed. If there is an entry and overwrite_keys is True, all  current
+    entries for the given hostname will be removed and the new keys added.
+
+    Args:
+      hostname: str, The hostname for the known_hosts entry.
+      host_keys: list, A list of host keys for the given hostname.
+      overwrite: bool, If true, will overwrite the entries corresponding to
+        hostname with the new host_key if it already exists. If false and an
+        entry already exists for hostname, will ignore the new host_key values.
+    Returns:
+      bool, True if new keys were added.
+    """
+    new_keys_added = False
+    new_key_entries = ['{0} {1}'.format(hostname, host_key)
+                       for host_key in host_keys]
+    if not new_key_entries:
+      return new_keys_added
+    existing_entries = [key for key in self.known_hosts
+                        if key.startswith(hostname)]
+    if existing_entries:
+      if overwrite:
+        self.known_hosts = [key for key in self.known_hosts
+                            if not key.startswith(hostname)]
+        self.known_hosts.extend(new_key_entries)
+        new_keys_added = True
+    else:
+      self.known_hosts.extend(new_key_entries)
+      new_keys_added = True
+
+    return new_keys_added
+
   def Write(self):
     """Writes the file to disk."""
     files.WriteFileContents(
@@ -728,12 +761,13 @@ def CheckForOsloginAndGetUser(instance, project, requested_user, public_key,
     elif pa.primary:
       oslogin_user = pa.username
 
-  log.warning('Using OS Login user [{0}] instead of default user [{1}]'
-              .format(oslogin_user, requested_user))
+  log.out.Print('Using OS Login user [{0}] instead of default user [{1}]'
+                .format(oslogin_user, requested_user))
   return oslogin_user, use_oslogin
 
 
-def ParseAndSubstituteSSHFlags(args, remote, ip_address):
+def ParseAndSubstituteSSHFlags(args, remote, instance_address,
+                               internal_address):
   """Obtain extra flags from the command arguments."""
   extra_flags = []
   if args.ssh_flag:
@@ -741,7 +775,8 @@ def ParseAndSubstituteSSHFlags(args, remote, ip_address):
       for flag_part in flag.split():  # We want grouping here
         dereferenced_flag = (
             flag_part.replace('%USER%', remote.user)
-            .replace('%INSTANCE%', ip_address))
+            .replace('%INSTANCE%', instance_address)
+            .replace('%INTERNAL%', internal_address))
         extra_flags.append(dereferenced_flag)
   return extra_flags
 
