@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -253,6 +253,8 @@ class _Sections(object):
     core: Section, The section containing core properties for the Cloud SDK.
     dataproc: Section, The section containing dataproc properties for the Cloud
       SDK.
+    datafusion: Section, The section containing datafusion properties for the
+      Cloud SDK.
     default_section: Section, The main section of the properties file (core).
     deployment_manager: Section, The section containing deployment_manager
       properties for the Cloud SDK.
@@ -268,6 +270,8 @@ class _Sections(object):
       Cloud SDK.
     functions: Section, The section containing functions properties for the
       Cloud SDK.
+    game_services: Section, The section containing gameservices properties for
+      the Cloud SDK.
     gcloudignore: Section, The section containing gcloudignore properties for
       the Cloud SDK.
     healthcare: Section, The section containing healthcare properties for the
@@ -286,6 +290,8 @@ class _Sections(object):
       SDK.
     storage: Section, The section containing storage properties for the Cloud
       SDK.
+    survey: Section, The section containing survey properties for the Cloud
+      SDK.
     test: Section, The section containing test properties for the Cloud SDK.
   """
 
@@ -310,6 +316,7 @@ class _Sections(object):
     self.container = _SectionContainer()
     self.core = _SectionCore()
     self.dataproc = _SectionDataproc()
+    self.datafusion = _SectionDatafusion()
     self.deployment_manager = _SectionDeploymentManager()
     self.devshell = _SectionDevshell()
     self.diagnostics = _SectionDiagnostics()
@@ -317,6 +324,7 @@ class _Sections(object):
     self.experimental = _SectionExperimental()
     self.filestore = _SectionFilestore()
     self.functions = _SectionFunctions()
+    self.game_services = _SectionGameServices()
     self.gcloudignore = _SectionGcloudignore()
     self.healthcare = _SectionHealthcare()
     self.interactive = _SectionInteractive()
@@ -328,6 +336,7 @@ class _Sections(object):
     self.run = _SectionRun()
     self.spanner = _SectionSpanner()
     self.storage = _SectionStorage()
+    self.survey = _SectionSurvey()
     self.test = _SectionTest()
 
     sections = [
@@ -345,6 +354,7 @@ class _Sections(object):
         self.container,
         self.core,
         self.dataproc,
+        self.datafusion,
         self.deployment_manager,
         self.devshell,
         self.diagnostics,
@@ -352,6 +362,7 @@ class _Sections(object):
         self.experimental,
         self.filestore,
         self.functions,
+        self.game_services,
         self.gcloudignore,
         self.healthcare,
         self.interactive,
@@ -361,6 +372,7 @@ class _Sections(object):
         self.redis,
         self.run,
         self.spanner,
+        self.survey,
         self.test,
     ]
     self.__sections = {section.name: section for section in sections}
@@ -697,6 +709,11 @@ class _SectionRun(_Section):
         'cluster_location',
         help_text='Zone or region of the GKE cluster to use.')
 
+    self.platform = self._Add(
+        'platform',
+        choices=['gke', 'managed', 'kubernetes'],
+        help_text='Target platform for running commands.')
+
 
 class _SectionSpanner(_Section):
   """Contains the properties for the 'spanner' section."""
@@ -790,6 +807,20 @@ class _SectionHealthcare(_Section):
         'command will fall back to this value, if set.')
 
 
+class _SectionGameServices(_Section):
+  """Contains the properties for the 'game_services' section."""
+
+  def __init__(self):
+    super(_SectionGameServices, self).__init__(
+        'game_services')
+    self.location = self._Add(
+        'location',
+        default='us-central1',
+        help_text='Default location to use when working with Cloud Game '
+        'Services resources. When a `--location` flag is required but not '
+        'provided, the command will fall back to this value.')
+
+
 class _SectionAccessibility(_Section):
   """Contains the properties for the 'accessibility' section."""
 
@@ -881,7 +912,8 @@ class _SectionBuilds(_Section):
     self.timeout = self._Add(
         'timeout',
         validator=_BuildTimeoutValidator,
-        help_text='Timeout, in seconds, to wait for builds to complete.')
+        help_text='Timeout, in seconds, to wait for builds to complete. If '
+        'unset, defaults to 10 minutes.')
     self.check_tag = self._AddBool(
         'check_tag',
         default=True,
@@ -926,20 +958,6 @@ class _SectionContainer(_Section):
         default=False,
         help_text='If True, use application default credentials to authenticate'
         ' to the cluster API server.')
-    self.new_scopes_behavior = self._AddBool(
-        'new_scopes_behavior',
-        default=False,
-        help_text='If True, use new scopes behavior and do not add `compute-rw`'
-        ', `storage-ro`, `service-control`, or `service-management` scopes. The'
-        ' former two (`compute-rw` and `storage-ro`) only apply to clusters '
-        'at Kubernetes v1.9 and below; starting v1.10, `compute-rw` and '
-        '`storage-ro` are not added by default. Any of these scopes '
-        'may be added explicitly using `--scopes`. Using new scopes behavior '
-        'will be the default in a future release. Additionally, if '
-        'this property is set to True, using `--[no-]enable-cloud-endpoints` '
-        'is not allowed. This property is ignored in alpha and beta, since '
-        'these tracks always use the new behavior. See `--scopes` help for '
-        'more info.')
 
     self.build_timeout = self._Add(
         'build_timeout',
@@ -995,7 +1013,7 @@ class _SectionCore(_Section):
     self.disable_usage_reporting = self._AddBool(
         'disable_usage_reporting',
         help_text='If True, anonymous statistics on SDK usage will not be '
-        'collected.  This value is set by default based on your choices during '
+        'collected. This value is set by default based on your choices during '
         'installation, but can be changed at any time.  For more information, '
         'see: https://cloud.google.com/sdk/usage-statistics')
     self.enable_gri = self._AddBool(
@@ -1155,6 +1173,12 @@ class _SectionCore(_Section):
         ' files. If unset, the default is 30 days.',
         default='30')
 
+    self.disable_file_logging = self._AddBool(
+        'disable_file_logging',
+        default=False,
+        help_text='If True, `gcloud` will not store logs to a file. This may '
+        'be useful if disk space is limited.')
+
     def ExistingAbsoluteFilepathValidator(file_path):
       """Checks to see if the file path exists and is an absolute path."""
       if file_path is None:
@@ -1381,6 +1405,22 @@ class _SectionComposer(_Section):
             'referenced.'))
 
 
+class _SectionDatafusion(_Section):
+  """Contains the properties for the 'datafusion' section."""
+
+  def __init__(self):
+    super(_SectionDatafusion, self).__init__('datafusion')
+    self.location = self._Add(
+        'location',
+        help_text=(
+            'Datafusion location to use. Each Datafusion location '
+            'constitutes an independent resource namespace constrained to '
+            'deploying environments into Compute Engine regions inside this '
+            'location. This parameter corresponds to the '
+            '/locations/<location> segment of the Datafusion resource URIs being '
+            'referenced.'))
+
+
 class _SectionDataproc(_Section):
   """Contains the properties for the 'ml_engine' section."""
 
@@ -1580,7 +1620,9 @@ class _SectionApiEndpointOverrides(_Section):
     self.compute = self._Add('compute')
     self.container = self._Add('container')
     self.containeranalysis = self._Add('containeranalysis')
+    self.datacatalog = self._Add('datacatalog')
     self.dataflow = self._Add('dataflow')
+    self.datafusion = self._Add('datafusion')
     self.datapol = self._Add('datapol')
     self.dataproc = self._Add('dataproc')
     self.datastore = self._Add('datastore')
@@ -1590,6 +1632,7 @@ class _SectionApiEndpointOverrides(_Section):
     self.domains = self._Add('domains')
     self.file = self._Add('file')
     self.firestore = self._Add('firestore')
+    self.gameservices = self._Add('gameservices')
     self.genomics = self._Add('genomics')
     self.gkehub = self._Add('gkehub')
     self.healthcare = self._Add('healthcare')
@@ -1602,6 +1645,7 @@ class _SectionApiEndpointOverrides(_Section):
     self.monitoring = self._Add('monitoring')
     self.oslogin = self._Add('oslogin')
     self.pubsub = self._Add('pubsub')
+    self.recommender = self._Add('recommender')
     self.replicapoolupdater = self._Add('replicapoolupdater')
     self.runtimeconfig = self._Add('runtimeconfig')
     self.redis = self._Add('redis')
@@ -1715,6 +1759,18 @@ class _SectionStorage(_Section):
         'uses gsutil.')
 
 
+class _SectionSurvey(_Section):
+  """Contains the properties for the 'survey' section."""
+
+  def __init__(self):
+    super(_SectionSurvey, self).__init__('survey')
+    self.disable_prompts = self._AddBool(
+        'disable_prompts',
+        default=False,
+        help_text='If True, gcloud will not prompt you to take periodic usage '
+        'experience surveys.')
+
+
 class _Property(object):
   """An individual property that can be gotten from the properties file.
 
