@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*- #
 #
-# Copyright 2013 Google Inc. All Rights Reserved.
+# Copyright 2013 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@ import errno
 import os
 import sys
 
+from googlecloudsdk.api_lib.iamcredentials import util as iamcred_util
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import cli
 from googlecloudsdk.command_lib import crash_handling
@@ -39,6 +40,7 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core import metrics
 from googlecloudsdk.core import properties
 from googlecloudsdk.core.credentials import store as creds_store
+from googlecloudsdk.core.survey import survey_check
 from googlecloudsdk.core.updater import local_state
 from googlecloudsdk.core.updater import update_manager
 from googlecloudsdk.core.util import keyboard_interrupt
@@ -64,6 +66,18 @@ def UpdateCheck(command_path, **unused_kwargs):
     log.debug('Failed to perform update check.', exc_info=True)
 
 
+def SurveyPromptCheck(command_path, **unused_kwargs):
+  del command_path
+  try:
+    if not properties.VALUES.survey.disable_prompts.GetBool():
+      survey_check.SurveyPrompter().PromptForSurvey()
+  # pylint:disable=broad-except, We never want this to escape, ever. Only
+  # messages printed should reach the user.
+  except Exception:
+    log.debug('Failed to check survey prompt.', exc_info=True)
+  # pylint:enable=broad-except
+
+
 def CreateCLI(surfaces, translator=None):
   """Generates the gcloud CLI from 'surface' folder with extra surfaces.
 
@@ -78,8 +92,8 @@ def CreateCLI(surfaces, translator=None):
   def VersionFunc():
     generated_cli.Execute(['version'])
 
-  def HandleKnownErrorFunc(exc):
-    crash_handling.ReportError(exc, is_crash=False)
+  def HandleKnownErrorFunc():
+    crash_handling.ReportError(is_crash=False)
 
   pkg_root = os.path.dirname(os.path.dirname(surface.__file__))
   loader = cli.CLILoader(
@@ -101,13 +115,30 @@ def CreateCLI(surfaces, translator=None):
   for dot_path, dir_path in surfaces:
     loader.AddModule(dot_path, dir_path, component=None)
 
+  # TODO(b/128465608): Remove cloned ml-engine commands and PreRunHook after a
+  # suitable deprecation period.
+  # Clone 'ai-platform' surface into 'ml-engine' for backward compatibility.
+  loader.AddModule('ml_engine', os.path.join(pkg_root, 'surface',
+                                             'ai_platform'))
+  loader.RegisterPreRunHook(
+      _IssueAIPlatformAliasWarning, include_commands=r'gcloud\..*ml-engine\..*')
+
   # Check for updates on shutdown but not for any of the updater commands.
-  loader.RegisterPostRunHook(UpdateCheck,
-                             exclude_commands=r'gcloud\.components\..*')
+  # Skip update checks for 'gcloud version' command as it does that manually.
+  exclude_commands = r'gcloud\.components\..*|gcloud\.version'
+  loader.RegisterPostRunHook(UpdateCheck, exclude_commands=exclude_commands)
+  loader.RegisterPostRunHook(SurveyPromptCheck)
   generated_cli = loader.Generate()
   return generated_cli
 
 
+def _IssueAIPlatformAliasWarning(command_path=None):
+  del command_path  # Unused in _IssueTestWarning
+  log.warning(
+      'The `gcloud ml-engine` commands have been renamed and will soon be '
+      'removed. Please use `gcloud ai-platform` instead.')
+
+
 def main(gcloud_cli=None, credential_providers=None):
   if not platforms.PythonVersion().IsCompatible(
       allow_py3=properties.VALUES.core.allow_py3.GetBool()):
@@ -127,6 +158,9 @@ def main(gcloud_cli=None, credential_providers=None):
   ]
   for provider in credential_providers:
     provider.Register()
+  # Register support for service account impersonation.
+  creds_store.IMPERSONATION_TOKEN_PROVIDER = (
+      iamcred_util.ImpersonationAccessTokenProvider())
 
   try:
     try:
