@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ from googlecloudsdk.core import log
 
 
 class UpdateCluster(base.UpdateCommand):
-  """Update a Bigtable cluster's friendly name and serving nodes."""
+  """Update a Bigtable cluster's number of nodes."""
 
   @staticmethod
   def Args(parser):
