@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ class DiskTypes(base.Group):
   """Read Google Compute Engine virtual disk types."""
 
 
-DiskTypes.category = base.COMPUTE_DISKS_CATEGORY
+DiskTypes.category = base.DISKS_CATEGORY
 
 DiskTypes.detailed_help = {
     'brief': 'Read Google Compute Engine virtual disk types',
