@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,10 +24,12 @@ from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute import scope as compute_scope
 from googlecloudsdk.command_lib.compute.instance_groups import flags as instance_groups_flags
+from googlecloudsdk.command_lib.compute.instance_groups.flags import AutoDeleteFlag
 from googlecloudsdk.command_lib.compute.managed_instance_groups import auto_healing_utils
+import six
 
 
-@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
+@base.ReleaseTracks(base.ReleaseTrack.GA)
 class UpdateGA(base.UpdateCommand):
   r"""Update Google Compute Engine managed instance groups.
 
@@ -120,8 +122,69 @@ class UpdateGA(base.UpdateCommand):
               autoHealingPolicies=auto_healing_policies))
 
 
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
+class UpdateBeta(UpdateGA):
+  r"""Update Google Compute Engine managed instance groups.
+
+  *{command}* allows you to specify or modify AutoHealingPolicy for an existing
+  managed instance group.
+
+  When updating the AutoHealingPolicy, you may specify the health check, initial
+  delay, or both. If the field is unspecified, its value won't be modified. If
+  `--health-check` is specified, the health check will be used to monitor the
+  health of your application. Whenever the health check signal for the instance
+  becomes `UNHEALTHY`, the autohealing action (`RECREATE`) on an instance will
+  be performed.
+
+  If no health check is specified, the instance autohealing will be triggered by
+  the instance status only (i.e. the autohealing action (`RECREATE`) on an
+  instance will be performed if `instance.status` is not `RUNNING`).
+  """
+
+  @staticmethod
+  def Args(parser):
+    UpdateGA.Args(parser)
+    instance_groups_flags.AddMigInstanceRedistributionTypeFlag(parser)
+
+  def Run(self, args):
+    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
+    client = holder.client
+    igm_ref = (instance_groups_flags.MULTISCOPE_INSTANCE_GROUP_MANAGER_ARG
+               .ResolveAsResource)(
+                   args,
+                   holder.resources,
+                   default_scope=compute_scope.ScopeEnum.ZONE,
+                   scope_lister=flags.GetDefaultScopeLister(client))
+
+    if igm_ref.Collection() not in [
+        'compute.instanceGroupManagers', 'compute.regionInstanceGroupManagers'
+    ]:
+      raise ValueError('Unknown reference type {0}'.format(
+          igm_ref.Collection()))
+
+    instance_groups_flags.ValidateMigInstanceRedistributionTypeFlag(
+        args.GetValue('instance_redistribution_type'), igm_ref)
+
+    igm_resource = managed_instance_groups_utils.GetInstanceGroupManagerOrThrow(
+        igm_ref, client)
+
+    update_policy = (managed_instance_groups_utils
+                     .ApplyInstanceRedistributionTypeToUpdatePolicy)(
+                         client, args.GetValue('instance_redistribution_type'),
+                         igm_resource.updatePolicy)
+
+    auto_healing_policies = self._GetValidatedAutohealingPolicies(
+        holder, client, args, igm_resource)
+
+    igm_updated_resource = client.messages.InstanceGroupManager(
+        updatePolicy=update_policy)
+    if auto_healing_policies is not None:
+      igm_updated_resource.autoHealingPolicies = auto_healing_policies
+    return self._MakePatchRequest(client, igm_ref, igm_updated_resource)
+
+
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
-class UpdateAlpha(UpdateGA):
+class UpdateAlpha(UpdateBeta):
   r"""Update Google Compute Engine managed instance groups.
 
   *{command}* allows you to specify or modify the StatefulPolicy and
@@ -147,38 +210,54 @@ class UpdateAlpha(UpdateGA):
 
   @staticmethod
   def Args(parser):
-    UpdateGA.Args(parser)
+    UpdateBeta.Args(parser)
     instance_groups_flags.AddMigUpdateStatefulFlags(parser)
-    instance_groups_flags.AddMigInstanceRedistributionTypeFlag(parser)
 
-  def _MakePreservedStateWithDisks(self, client, device_names):
+  def _MakePreservedStateDiskEntry(self, client, stateful_disk_dict):
     """Create StatefulPolicyPreservedState from a list of device names."""
-    additional_properties = []
-    # Disk device with AutoDelete NEVER
+    auto_delete = (stateful_disk_dict.get('auto-delete') or
+                   AutoDeleteFlag.NEVER).GetAutoDeleteEnumValue(
+                       client.messages.StatefulPolicyPreservedStateDiskDevice
+                       .AutoDeleteValueValuesEnum)
     disk_device = client.messages.StatefulPolicyPreservedStateDiskDevice(
-        autoDelete=
-        client.messages.StatefulPolicyPreservedStateDiskDevice \
-          .AutoDeleteValueValuesEnum.NEVER)
+        autoDelete=auto_delete)
     # Add all disk_devices to map
-    for device_name in device_names:
-      disk_value = client.messages.StatefulPolicyPreservedState.DisksValue \
-        .AdditionalProperty(key=device_name, value=disk_device)
-      additional_properties.append(disk_value)
-    return client.messages.StatefulPolicyPreservedState(
-        disks=client.messages.StatefulPolicyPreservedState.DisksValue(
-            additionalProperties=additional_properties))
+    return client.messages.StatefulPolicyPreservedState.DisksValue \
+        .AdditionalProperty(
+            key=stateful_disk_dict.get('device-name'), value=disk_device)
 
-  def _UpdateStatefulPolicy(self, client, device_names):
-    preserved_disks = [
-        client.messages.StatefulPolicyPreservedDisk(deviceName=device_name)
-        for device_name in device_names
-    ]
-    preserved_state = self._MakePreservedStateWithDisks(client, device_names)
-    if preserved_disks:
+  def _UpdateStatefulPolicy(self, client, current_stateful_policy, update_disks,
+                            remove_device_names):
+    """Create an updated stateful policy with the updated disk data and removed disks as specified."""
+    if not update_disks:
+      update_disks = []
+    if not remove_device_names:
+      remove_device_names = []
+    update_map = {
+        update_disk.get('device-name'): update_disk
+        for update_disk in update_disks
+    }
+    additional_properties = []
+    if current_stateful_policy and current_stateful_policy.preservedState \
+        and current_stateful_policy.preservedState.disks:
+      current_disks = current_stateful_policy\
+        .preservedState.disks.additionalProperties
+    else:
+      current_disks = []
+    for disk_entry in current_disks:
+      if disk_entry.key in remove_device_names:
+        continue
+      if disk_entry.key not in update_map:
+        additional_properties.append(disk_entry)
+    for _, stateful_disk in six.iteritems(update_map):
+      additional_properties.append(
+          self._MakePreservedStateDiskEntry(client, stateful_disk))
+    additional_properties.sort(key=lambda x: x.key)
+    if additional_properties:
       return client.messages.StatefulPolicy(
-          preservedResources=client.messages.StatefulPolicyPreservedResources(
-              disks=preserved_disks),
-          preservedState=preserved_state)
+          preservedState=client.messages.StatefulPolicyPreservedState(
+              disks=client.messages.StatefulPolicyPreservedState.DisksValue(
+                  additionalProperties=additional_properties)))
     else:
       return client.messages.StatefulPolicy()
 
@@ -201,9 +280,13 @@ class UpdateAlpha(UpdateGA):
 
   def _StatefulArgsSet(self, args):
     return (args.IsSpecified('stateful_names') or
-            args.IsSpecified('add_stateful_disks') or
+            args.IsSpecified('update_stateful_disk') or
             args.IsSpecified('remove_stateful_disks'))
 
+  def _StatefulnessIntroduced(self, args):
+    return (args.IsSpecified('stateful_names') or
+            args.IsSpecified('update_stateful_disk'))
+
   def Run(self, args):
     holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
     client = holder.client
@@ -225,8 +308,11 @@ class UpdateAlpha(UpdateGA):
 
     igm_resource = managed_instance_groups_utils.GetInstanceGroupManagerOrThrow(
         igm_ref, client)
+    if self._StatefulnessIntroduced(args):
+      managed_instance_groups_utils.ValidateIgmReadyForStatefulness(
+          igm_resource, client)
 
-    device_names = instance_groups_flags.GetValidatedUpdateStatefulPolicyParams(
+    device_names = instance_groups_flags.ValidateUpdateStatefulPolicyParams(
         args, igm_resource.statefulPolicy)
 
     update_policy = (managed_instance_groups_utils
@@ -250,13 +336,17 @@ class UpdateAlpha(UpdateGA):
           'stateful_names') and not args.GetValue('stateful_names'):
         igm_resource.reset('statefulPolicy')
       elif igm_resource.statefulPolicy or args.GetValue('stateful_names'):
-        igm_resource.statefulPolicy = self._UpdateStatefulPolicy(client, [])
+        igm_resource.statefulPolicy = self._UpdateStatefulPolicy(
+            client, igm_resource.statefulPolicy, args.update_stateful_disk,
+            args.remove_stateful_disks)
       igm_resource.updatePolicy = update_policy
       if auto_healing_policies is not None:
         igm_resource.autoHealingPolicies = auto_healing_policies
       return self._MakeUpdateRequest(client, igm_ref, igm_resource)
 
-    stateful_policy = self._UpdateStatefulPolicy(client, device_names)
+    stateful_policy = self._UpdateStatefulPolicy(
+        client, igm_resource.statefulPolicy, args.update_stateful_disk,
+        args.remove_stateful_disks)
     igm_updated_resource = client.messages.InstanceGroupManager(
         statefulPolicy=stateful_policy, updatePolicy=update_policy)
     if auto_healing_policies is not None:
