@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,6 +30,7 @@ from googlecloudsdk.api_lib.compute.operations import poller
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import completers
+from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
 from googlecloudsdk.command_lib.compute.resource_policies import flags as maintenance_flags
 from googlecloudsdk.command_lib.compute.resource_policies import util as maintenance_util
@@ -63,7 +64,7 @@ DETAILED_HELP = {
         To create an instance with the latest ``Red Hat Enterprise Linux
         7'' image available, run:
 
-          $ {command} example-instance --image-family rhel-7 --image-project rhel-cloud --zone us-central1-a
+          $ {command} example-instance --image-family=rhel-7 --image-project=rhel-cloud --zone=us-central1-a
         """,
 }
 
@@ -74,12 +75,16 @@ def _CommonArgs(parser,
                 enable_snapshots=False,
                 deprecate_maintenance_policy=False,
                 supports_display_device=False,
-                supports_reservation=False):
+                supports_reservation=False,
+                enable_resource_policy=False,
+                csek_key_file=False):
   """Register parser args common to all tracks."""
   metadata_utils.AddMetadataArgs(parser)
   instances_flags.AddDiskArgs(parser, enable_regional, enable_kms=enable_kms)
   instances_flags.AddCreateDiskArgs(parser, enable_kms=enable_kms,
-                                    enable_snapshots=enable_snapshots)
+                                    enable_snapshots=enable_snapshots,
+                                    resource_policy=enable_resource_policy,
+                                    csek_key_file=csek_key_file)
   instances_flags.AddCanIpForwardArgs(parser)
   instances_flags.AddAddressArgs(parser, instances=True)
   instances_flags.AddAcceleratorArgs(parser)
@@ -142,6 +147,9 @@ class Create(base.CreateCommand):
   _support_snapshots = False
   _support_display_device = False
   _support_reservation = False
+  _support_disk_resource_policy = False
+  _support_erase_vss = False
+  _support_machine_image_key = False
 
   @classmethod
   def Args(cls, parser):
@@ -223,7 +231,8 @@ class Create(base.CreateCommand):
               getattr(args, 'create_disk', []),
               instance_ref,
               enable_kms=self._support_kms,
-              enable_snapshots=self._support_snapshots))
+              enable_snapshots=self._support_snapshots,
+              resource_policy=self._support_disk_resource_policy))
       local_nvdimms = []
       if self._support_nvdimm:
         local_nvdimms = instance_utils.CreateLocalNvdimmMessages(
@@ -433,6 +442,10 @@ class Create(base.CreateCommand):
       if shielded_instance_config:
         instance.shieldedInstanceConfig = shielded_instance_config
 
+      if self._support_erase_vss and \
+        args.IsSpecified('erase_windows_vss_signature'):
+        instance.eraseWindowsVssSignature = args.erase_windows_vss_signature
+
       request = compute_client.messages.ComputeInstancesInsertRequest(
           instance=instance,
           project=instance_ref.project,
@@ -443,6 +456,18 @@ class Create(base.CreateCommand):
 
       if source_machine_image:
         request.instance.sourceMachineImage = source_machine_image
+        if args.IsSpecified('source_machine_image_csek_key_file'):
+          key = instance_utils.GetSourceMachineImageKey(
+              args, self.SOURCE_MACHINE_IMAGE, compute_client, holder)
+          request.instance.sourceMachineImageEncryptionKey = key
+
+      if self._support_machine_image_key and \
+          args.IsSpecified('source_machine_image_csek_key_file'):
+        if not args.IsSpecified('source_machine_image'):
+          raise exceptions.RequiredArgumentException(
+              '`--source-machine-image`',
+              '`--source-machine-image-csek-key-file` requires '
+              '`--source-machine-image` to be specified`')
 
       if (self._support_display_device and
           args.IsSpecified('enable_display_device')):
@@ -527,9 +552,12 @@ class CreateBeta(Create):
   _support_kms = True
   _support_nvdimm = False
   _support_public_dns = False
-  _support_snapshots = False
+  _support_snapshots = True
   _support_display_device = True
-  _support_reservation = False
+  _support_reservation = True
+  _support_disk_resource_policy = True
+  _support_erase_vss = False
+  _support_machine_image_key = False
 
   def _GetNetworkInterfaces(
       self, args, client, holder, instance_refs, skip_defaults):
@@ -542,7 +570,10 @@ class CreateBeta(Create):
         parser,
         enable_regional=True,
         enable_kms=True,
-        supports_display_device=True
+        enable_snapshots=True,
+        supports_display_device=True,
+        supports_reservation=cls._support_reservation,
+        enable_resource_policy=cls._support_disk_resource_policy,
     )
     cls.SOURCE_INSTANCE_TEMPLATE = (
         instances_flags.MakeSourceInstanceTemplateArg())
@@ -561,6 +592,9 @@ class CreateAlpha(CreateBeta):
   _support_snapshots = True
   _support_display_device = True
   _support_reservation = True
+  _support_disk_resource_policy = True
+  _support_erase_vss = True
+  _support_machine_image_key = True
 
   def _GetNetworkInterfaces(
       self, args, client, holder, instance_refs, skip_defaults):
@@ -582,17 +616,22 @@ class CreateAlpha(CreateBeta):
         enable_snapshots=True,
         deprecate_maintenance_policy=True,
         supports_display_device=True,
-        supports_reservation=cls._support_reservation)
+        supports_reservation=cls._support_reservation,
+        enable_resource_policy=cls._support_disk_resource_policy,
+        csek_key_file=True)
     CreateAlpha.SOURCE_INSTANCE_TEMPLATE = (
         instances_flags.MakeSourceInstanceTemplateArg())
     CreateAlpha.SOURCE_INSTANCE_TEMPLATE.AddArgument(parser)
     CreateAlpha.SOURCE_MACHINE_IMAGE = (
         instances_flags.AddMachineImageArg())
     CreateAlpha.SOURCE_MACHINE_IMAGE.AddArgument(parser)
+    instances_flags.AddSourceMachineImageEncryptionKey(parser)
     instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.ALPHA)
     instances_flags.AddPublicDnsArgs(parser, instance=True)
     instances_flags.AddLocalSsdArgsWithSize(parser)
     instances_flags.AddLocalNvdimmArgs(parser)
+    flags.AddEraseVssSignature(parser, 'source snapshots or source machine'
+                                       ' image')
     maintenance_flags.AddResourcePoliciesArgs(parser, 'added to', 'instance')
 
 Create.detailed_help = DETAILED_HELP
