@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2014 Google Inc. All Rights Reserved.
+# Copyright 2014 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,11 +36,6 @@ from googlecloudsdk.core import log
 from googlecloudsdk.core.util import retry
 
 
-def ArgsHaveTunnelThroughIap(args):
-  """Determine if the current track has this flag and if it is also enabled."""
-  return hasattr(args, 'tunnel_through_iap') and args.tunnel_through_iap
-
-
 def AddCommandArg(parser):
   parser.add_argument(
       '--command',
@@ -67,6 +62,9 @@ def AddSSHArgs(parser):
       is equivalent to passing the flags ``--vvv'' and ``-L
       80:162.222.181.197:80'' to *ssh(1)* if the external IP address of
       'example-instance' is 162.222.181.197.
+
+      If connecting to the instance's external IP, then %INSTANCE% is replaced
+      with that, otherwise it is replaced with the internal IP.
       """)
 
   parser.add_argument(
@@ -77,7 +75,7 @@ def AddSSHArgs(parser):
       Specifies the instance to SSH into.
 
       ``USER'' specifies the username with which to SSH. If omitted,
-      $USER from the environment is selected.
+      the user login name is used.
 
       ``INSTANCE'' specifies the name of the virtual machine instance to SSH
       into.
@@ -130,6 +128,7 @@ class Ssh(base.Command):
   """SSH into a virtual machine instance."""
 
   category = base.TOOLS_CATEGORY
+  get_host_keys = False
 
   @staticmethod
   def Args(parser):
@@ -160,26 +159,54 @@ class Ssh(base.Command):
         scope_lister=instance_flags.GetInstanceZoneScopeLister(client))[0]
     instance = ssh_helper.GetInstance(client, instance_ref)
     project = ssh_helper.GetProject(client, instance_ref.project)
+    if self.get_host_keys:
+      host_keys = ssh_helper.GetHostKeysFromGuestAttributes(
+          client, instance_ref)
+      if not host_keys:
+        log.warning('Unable to retrieve host keys from instance metadata. '
+                    'Continuing.')
+    else:
+      host_keys = {}
     if args.plain:
       use_oslogin = False
     else:
       public_key = ssh_helper.keys.GetPublicKey().ToEntry(include_comment=True)
       user, use_oslogin = ssh.CheckForOsloginAndGetUser(
           instance, project, user, public_key, self.ReleaseTrack())
-    if args.internal_ip or ArgsHaveTunnelThroughIap(args):
-      ip_address = ssh_utils.GetInternalIPAddress(instance)
+
+    iap_tunnel_args = iap_tunnel.SshTunnelArgs.FromArgs(
+        args, self.ReleaseTrack(), instance_ref,
+        ssh_utils.GetInternalInterface(instance),
+        ssh_utils.GetExternalInterface(instance, no_raise=True))
+
+    internal_address = ssh_utils.GetInternalIPAddress(instance)
+
+    if iap_tunnel_args:
+      # IAP Tunnel only uses instance_address for the purpose of --ssh-flag
+      # substitution. In this case, dest_addr doesn't do much, it just matches
+      # against entries in the user's ssh_config file. It's best to use
+      # something unique to avoid false positive matches, thus we use
+      # HostKeyAlias.
+      instance_address = internal_address
+      dest_addr = ssh_utils.HostKeyAlias(instance)
+    elif args.internal_ip:
+      instance_address = internal_address
+      dest_addr = instance_address
     else:
-      ip_address = ssh_utils.GetExternalIPAddress(instance)
-    remote = ssh.Remote(ip_address, user)
+      instance_address = ssh_utils.GetExternalIPAddress(instance)
+      dest_addr = instance_address
+    remote = ssh.Remote(dest_addr, user)
 
     identity_file = None
     options = None
     if not args.plain:
       identity_file = ssh_helper.keys.key_file
       options = ssh_helper.GetConfig(ssh_utils.HostKeyAlias(instance),
-                                     args.strict_host_key_checking)
+                                     args.strict_host_key_checking,
+                                     host_keys_to_add=host_keys)
 
-    extra_flags = ssh.ParseAndSubstituteSSHFlags(args, remote, ip_address)
+    extra_flags = ssh.ParseAndSubstituteSSHFlags(args, remote, instance_address,
+                                                 internal_address)
     remainder = []
 
     if args.ssh_args:
@@ -198,22 +225,13 @@ class Ssh(base.Command):
                     'extra_flags': extra_flags,
                     'remote_command': remote_command,
                     'tty': tty,
+                    'iap_tunnel_args': iap_tunnel_args,
                     'remainder': remainder}
 
-    tunnel_helper = None
-    if ArgsHaveTunnelThroughIap(args):
-      tunnel_helper = ssh_utils.CreateIapTunnelHelper(args, instance_ref,
-                                                      instance)
-      tunnel_helper.StartListener()
-      ssh_cmd_args['remote'] = ssh.Remote('localhost', user)
-      ssh_cmd_args['port'] = str(tunnel_helper.GetLocalPort())
-
     cmd = ssh.SSHCommand(**ssh_cmd_args)
 
     if args.dry_run:
       log.out.Print(' '.join(cmd.Build(ssh_helper.env)))
-      if tunnel_helper:
-        tunnel_helper.StopListener()
       return
 
     if args.plain or use_oslogin:
@@ -223,50 +241,34 @@ class Ssh(base.Command):
           client, remote.user, instance, project)
 
     if keys_newly_added:
-      poller_tunnel_helper = None
-      if tunnel_helper:
-        poller_tunnel_helper = ssh_utils.CreateIapTunnelHelper(
-            args, instance_ref, instance)
-        poller_tunnel_helper.StartListener(accept_multiple_connections=True)
-      poller = ssh_utils.CreateSSHPoller(
-          remote, identity_file, options, poller_tunnel_helper,
-          extra_flags=extra_flags)
-
+      poller = ssh_utils.CreateSSHPoller(remote, identity_file, options,
+                                         iap_tunnel_args,
+                                         extra_flags=extra_flags)
       log.status.Print('Waiting for SSH key to propagate.')
       # TODO(b/35355795): Don't force_connect
       try:
         poller.Poll(ssh_helper.env, force_connect=True)
       except retry.WaitException:
-        if tunnel_helper:
-          tunnel_helper.StopListener()
         raise ssh_utils.NetworkError()
-      finally:
-        if poller_tunnel_helper:
-          poller_tunnel_helper.StopListener()
 
-    if args.internal_ip and not tunnel_helper:
-      # The IAP Tunnel connection uses instance name and network interface name,
-      # so do not need to additionally verify the instance.  Also, the
-      # SSHCommand used within the function does not support IAP Tunnels.
+    if args.internal_ip:
       ssh_helper.PreliminarilyVerifyInstance(instance.id, remote, identity_file,
                                              options)
 
-    try:
-      # Errors from SSH itself result in an ssh.CommandError being raised
-      return_code = cmd.Run(ssh_helper.env, force_connect=True)
-    finally:
-      if tunnel_helper:
-        tunnel_helper.StopListener()
+    # Errors from SSH itself result in an ssh.CommandError being raised
+    return_code = cmd.Run(ssh_helper.env, force_connect=True)
     if return_code:
       # This is the return code of the remote command.  Problems with SSH itself
       # will result in ssh.CommandError being raised above.
       sys.exit(return_code)
 
 
-@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
+@base.ReleaseTracks(base.ReleaseTrack.BETA)
 class SshBeta(Ssh):
   """SSH into a virtual machine instance (Beta)."""
 
+  get_host_keys = False
+
   @staticmethod
   def Args(parser):
     ssh_utils.BaseSSHCLIHelper.Args(parser)
@@ -278,7 +280,14 @@ class SshBeta(Ssh):
 
     mutex_scope = parser.add_mutually_exclusive_group()
     AddInternalIPArg(mutex_scope)
-    iap_tunnel.AddConnectionHelperArgs(parser, mutex_scope)
+    iap_tunnel.AddSshTunnelArgs(parser, mutex_scope)
+
+
+@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
+class SshAlpha(SshBeta):
+  """SSH into a virtual machine instance (Alpha)."""
+
+  get_host_keys = True
 
 
 def DetailedHelp():
