@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,14 +23,18 @@ from googlecloudsdk.api_lib.compute import utils
 from googlecloudsdk.api_lib.functions import env_vars as env_vars_api_util
 from googlecloudsdk.api_lib.functions import util as api_util
 from googlecloudsdk.calliope import base
+from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.functions import flags
 from googlecloudsdk.command_lib.functions.deploy import env_vars_util
 from googlecloudsdk.command_lib.functions.deploy import labels_util
 from googlecloudsdk.command_lib.functions.deploy import source_util
 from googlecloudsdk.command_lib.functions.deploy import trigger_util
+from googlecloudsdk.command_lib.util.apis import arg_utils
 from googlecloudsdk.command_lib.util.args import labels_util as args_labels_util
 from googlecloudsdk.command_lib.util.args import map_util
 from googlecloudsdk.core import log
+from googlecloudsdk.core import properties
+from googlecloudsdk.core.console import console_io
 
 
 def _ApplyEnvVarsArgsToFunction(function, args):
@@ -45,8 +49,25 @@ def _ApplyEnvVarsArgsToFunction(function, args):
   return updated_fields
 
 
-def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
-         enable_connected_vpc=False):
+def _CreateBindPolicyCommand(function_name, region):
+  template = (
+      'gcloud alpha functions add-iam-policy-binding %s %s'
+      '--member=allUsers --role=roles/cloudfunctions.invoker')
+  region_flag = '--region=%s ' % region if region else ''
+  return template % (function_name, region_flag)
+
+
+def _GetProject(args):
+  return args.project or properties.VALUES.core.project.Get(required=True)
+
+
+def _Run(args,
+         track=None,
+         enable_runtime=True,
+         enable_max_instances=False,
+         enable_vpc_connector=False,
+         enable_traffic_control=False,
+         enable_allow_unauthenticated=False):
   """Run a function deployment with the given args."""
   # Check for labels that start with `deployment`, which is not allowed.
   labels_util.CheckNoDeploymentLabels('--remove-labels', args.remove_labels)
@@ -56,7 +77,6 @@ def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
   trigger_util.ValidateTriggerArgs(
       args.trigger_event, args.trigger_resource,
       args.IsSpecified('retry'), args.IsSpecified('trigger_http'))
-
   trigger_params = trigger_util.GetTriggerEventParams(
       args.trigger_http, args.trigger_bucket, args.trigger_topic,
       args.trigger_event, args.trigger_resource)
@@ -69,6 +89,8 @@ def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
   # Get an existing function or create a new one.
   function = api_util.GetFunction(function_url)
   is_new_function = function is None
+  had_vpc_connector = bool(
+      function.vpcConnector) if not is_new_function else False
   if is_new_function:
     trigger_util.CheckTriggerSpecified(args)
     function = messages.CloudFunction()
@@ -99,23 +121,51 @@ def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
     if args.IsSpecified('runtime'):
       function.runtime = args.runtime
       updated_fields.append('runtime')
+      if args.runtime in ['nodejs', 'nodejs6']:  # nodejs is nodejs6 alias
+        log.warning(
+            'The Node.js 6 runtime is deprecated on Cloud Functions. '
+            'Please migrate to Node.js 8 (--runtime=nodejs8) or Node.js 10 '
+            '(--runtime=nodejs10). '
+            'See https://cloud.google.com/functions/docs/migrating/nodejs-runtimes'
+        )
     elif is_new_function:
-      log.warning('Flag `--runtime` will become a required flag soon. '
-                  'Please specify the value for this flag.')
+      raise exceptions.RequiredArgumentException(
+          'runtime', 'Flag `--runtime` is required for new functions.')
   if enable_max_instances:
     if (args.IsSpecified('max_instances') or
         args.IsSpecified('clear_max_instances')):
       max_instances = 0 if args.clear_max_instances else args.max_instances
       function.maxInstances = max_instances
       updated_fields.append('maxInstances')
-  if enable_connected_vpc:
-    if args.connected_vpc:
-      function.network = args.connected_vpc
-      updated_fields.append('network')
-    if args.vpc_connector:
-      function.vpcConnector = args.vpc_connector
+  if enable_vpc_connector:
+    if args.vpc_connector or args.clear_vpc_connector:
+      function.vpcConnector = ('' if args.clear_vpc_connector else
+                               args.vpc_connector)
       updated_fields.append('vpcConnector')
-
+  if enable_traffic_control:
+    if args.IsSpecified('egress_settings'):
+      will_have_vpc_connector = ((had_vpc_connector and
+                                  not args.clear_vpc_connector) or
+                                 args.vpc_connector)
+      if not will_have_vpc_connector:
+        raise exceptions.RequiredArgumentException(
+            'vpc-connector', 'Flag `--vpc-connector` is '
+            'required for setting `egress-settings`.')
+      egress_settings_enum = arg_utils.ChoiceEnumMapper(
+          arg_name='egress_settings',
+          message_enum=function.VpcConnectorEgressSettingsValueValuesEnum,
+          custom_mappings=flags.EGRESS_SETTINGS_MAPPING).GetEnumForChoice(
+              args.egress_settings)
+      function.vpcConnectorEgressSettings = egress_settings_enum
+      updated_fields.append('vpcConnectorEgressSettings')
+    if args.IsSpecified('ingress_settings'):
+      ingress_settings_enum = arg_utils.ChoiceEnumMapper(
+          arg_name='ingress_settings',
+          message_enum=function.IngressSettingsValueValuesEnum,
+          custom_mappings=flags.INGRESS_SETTINGS_MAPPING).GetEnumForChoice(
+              args.ingress_settings)
+      function.ingressSettings = ingress_settings_enum
+      updated_fields.append('ingressSettings')
   # Populate trigger properties of function based on trigger args.
   if args.trigger_http:
     function.httpsTrigger = messages.HttpsTrigger()
@@ -142,7 +192,8 @@ def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
   if (args.source or args.stage_bucket or is_new_function or
       function.sourceUploadUrl):
     updated_fields.extend(source_util.SetFunctionSourceProps(
-        function, function_ref, args.source, args.stage_bucket))
+        function, function_ref, args.source, args.stage_bucket,
+        args.ignore_file))
 
   # Apply label args to function
   if labels_util.SetFunctionLabels(function, args.update_labels,
@@ -152,12 +203,53 @@ def _Run(args, track=None, enable_runtime=True, enable_max_instances=False,
   # Apply environment variables args to function
   updated_fields.extend(_ApplyEnvVarsArgsToFunction(function, args))
 
+  ensure_all_users_invoke = (
+      enable_allow_unauthenticated and flags.ShouldEnsureAllUsersInvoke(args))
+  deny_all_users_invoke = (
+      enable_allow_unauthenticated and flags.ShouldDenyAllUsersInvoke(args))
+
   if is_new_function:
-    return api_util.CreateFunction(function,
-                                   function_ref.Parent().RelativeName())
-  if updated_fields:
-    return api_util.PatchFunction(function, updated_fields)
-  log.status.Print('Nothing to update.')
+    if (enable_allow_unauthenticated
+        and not ensure_all_users_invoke
+        and not deny_all_users_invoke
+        and api_util.CanAddFunctionIamPolicyBinding(_GetProject(args))):
+      ensure_all_users_invoke = console_io.PromptContinue(
+          prompt_string=(
+              'Allow unauthenticated invocations of new function [{}]?'.format(
+                  args.NAME)),
+          default=False)
+
+    op = api_util.CreateFunction(function, function_ref.Parent().RelativeName())
+    if (enable_allow_unauthenticated
+        and not ensure_all_users_invoke
+        and not deny_all_users_invoke):
+      template = (
+          'Function created with default IAM policy. '
+          'To enable unauthorized access consider "%s"')
+      log.warning(template % _CreateBindPolicyCommand(args.NAME, args.region))
+
+  elif updated_fields:
+    op = api_util.PatchFunction(function, updated_fields)
+
+  else:
+    op = None  # Nothing to wait for
+    if not ensure_all_users_invoke and not deny_all_users_invoke:
+      log.status.Print('Nothing to update.')
+      return
+
+  try:
+    if ensure_all_users_invoke:
+      api_util.AddFunctionIamPolicyBinding(function.name)
+    elif deny_all_users_invoke:
+      api_util.RemoveFunctionIamPolicyBindingIfFound(function.name)
+  except exceptions.HttpException:
+    log.warning(
+        'Setting IAM policy failed, try "%s"' % _CreateBindPolicyCommand(
+            args.NAME, args.region))
+
+  if op:
+    api_util.WaitForFunctionUpdateOperation(op)
+  return api_util.GetFunction(function.name)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.GA)
@@ -192,6 +284,9 @@ class Deploy(base.Command):
     # Add args for specifying environment variables
     env_vars_util.AddUpdateEnvVarsFlags(parser)
 
+    # Add args for specifying ignore files to upload source
+    flags.AddIgnoreFileFlag(parser)
+
   def Run(self, args):
     return _Run(args, track=self.ReleaseTrack())
 
@@ -205,9 +300,16 @@ class DeployBeta(base.Command):
     """Register flags for this command."""
     Deploy.Args(parser)
     flags.AddMaxInstancesFlag(parser)
+    flags.AddVPCConnectorMutexGroup(parser)
+    flags.AddAllowUnauthenticatedFlag(parser)
 
   def Run(self, args):
-    return _Run(args, track=self.ReleaseTrack(), enable_max_instances=True)
+    return _Run(
+        args,
+        track=self.ReleaseTrack(),
+        enable_max_instances=True,
+        enable_vpc_connector=True,
+        enable_allow_unauthenticated=True)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@@ -219,8 +321,16 @@ class DeployAlpha(base.Command):
     """Register flags for this command."""
     Deploy.Args(parser)
     flags.AddMaxInstancesFlag(parser)
-    flags.AddConnectedVPCMutexGroup(parser)
+    flags.AddVPCConnectorMutexGroup(parser)
+    flags.AddEgressSettingsFlag(parser)
+    flags.AddIngressSettingsFlag(parser)
+    flags.AddAllowUnauthenticatedFlag(parser)
 
   def Run(self, args):
-    return _Run(args, track=self.ReleaseTrack(), enable_max_instances=True,
-                enable_connected_vpc=True)
+    return _Run(
+        args,
+        track=self.ReleaseTrack(),
+        enable_max_instances=True,
+        enable_vpc_connector=True,
+        enable_traffic_control=True,
+        enable_allow_unauthenticated=True)
