@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,9 @@
 """Decrypt a ciphertext file using a key."""
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import unicode_literals
+
 from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
@@ -131,7 +133,13 @@ class Decrypt(base.Command):
     resp = client.projects_locations_keyRings_cryptoKeys.Decrypt(req)
 
     try:
-      log.WriteToFileOrStdout(
-          args.plaintext_file, resp.plaintext, binary=True, overwrite=True)
+      if resp.plaintext is None:
+        with files.FileWriter(args.plaintext_file):
+          # to create an empty file
+          pass
+        log.Print('Decrypted file is empty')
+      else:
+        log.WriteToFileOrStdout(
+            args.plaintext_file, resp.plaintext, binary=True, overwrite=True)
     except files.Error as e:
       raise exceptions.BadFileException(e)
