@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2015 Google Inc. All Rights Reserved.
+# Copyright 2015 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,9 @@
 """Command to create a new project."""
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import unicode_literals
+
 from apitools.base.py import exceptions as apitools_exceptions
 
 from googlecloudsdk.api_lib.cloudresourcemanager import projects_api
@@ -151,9 +153,9 @@ class Create(base.CreateCommand):
 
     if args.set_as_default:
       project_property = properties.FromString('core/project')
-      properties.PersistProperty(project_property, args.id)
+      properties.PersistProperty(project_property, project_id)
       log.status.Print('Updated property [core/project] to [{0}].'
-                       .format(args.id))
+                       .format(project_id))
 
     return operations.ExtractOperationResponse(create_op,
                                                apis.GetMessagesModule(
