@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2016 Google Inc. All Rights Reserved.
+# Copyright 2016 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -75,7 +75,7 @@ class UpdateAlphaBeta(base.UpdateCommand):
       result = client.Patch(
           subscription_ref,
           ack_deadline=args.ack_deadline,
-          push_config=util.ParsePushConfig(args.push_endpoint),
+          push_config=util.ParsePushConfig(args),
           retain_acked_messages=args.retain_acked_messages,
           labels=labels_update.GetOrNone(),
           message_retention_duration=args.message_retention_duration,
@@ -123,13 +123,22 @@ class UpdateGA(base.UpdateCommand):
     client = subscriptions.SubscriptionsClient()
     subscription_ref = args.CONCEPTS.subscription.Parse()
 
+    no_expiration = False
+    expiration_period = getattr(args, 'expiration_period', None)
+    if expiration_period:
+      if expiration_period == subscriptions.NEVER_EXPIRATION_PERIOD_VALUE:
+        no_expiration = True
+        expiration_period = None
+
     try:
       result = client.Patch(
           subscription_ref,
           ack_deadline=args.ack_deadline,
-          push_config=util.ParsePushConfig(args.push_endpoint),
+          push_config=util.ParsePushConfig(args),
           retain_acked_messages=args.retain_acked_messages,
-          message_retention_duration=args.message_retention_duration)
+          message_retention_duration=args.message_retention_duration,
+          no_expiration=no_expiration,
+          expiration_period=expiration_period)
     except subscriptions.NoFieldsSpecifiedError:
       raise
     else:
