@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2017 Google Inc. All Rights Reserved.
+# Copyright 2017 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,9 +16,11 @@
 """Cloud Pub/Sub topics update command."""
 
 from __future__ import absolute_import
+from __future__ import division
 from __future__ import unicode_literals
 
 from googlecloudsdk.api_lib.pubsub import topics
+from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.command_lib.pubsub import resource_args
 from googlecloudsdk.command_lib.util.args import labels_util
@@ -41,11 +43,21 @@ class Update(base.UpdateCommand):
     """Registers flags for this command."""
     resource_args.AddTopicResourceArg(parser, 'to update.')
     labels_util.AddUpdateLabelsFlags(parser)
-    parser.add_argument(
+
+    msp_group = parser.add_group(
+        mutex=True, help='Message storage policy options.')
+    msp_group.add_argument(
         '--recompute-message-storage-policy',
         action='store_true',
         help='If given, Cloud Pub/Sub will recompute the regions where messages'
-             " can be stored at rest based on your organization's policies.")
+        ' can be stored at rest, based on your organization\'s "Resource '
+        ' Location Restriction" policy.')
+    msp_group.add_argument(
+        '--message-storage-policy-allowed-regions',
+        metavar='REGION',
+        type=arg_parsers.ArgList(),
+        help='A list of one or more Cloud regions where messages are allowed to'
+        ' be stored at rest.')
 
   def Run(self, args):
     """This is what gets called when the user runs this command.
@@ -73,11 +85,15 @@ class Update(base.UpdateCommand):
       result = client.Patch(
           topic_ref,
           labels_update.GetOrNone(),
-          args.recompute_message_storage_policy)
+          args.recompute_message_storage_policy,
+          args.message_storage_policy_allowed_regions)
     except topics.NoFieldsSpecifiedError:
-      if not any(args.IsSpecified(arg) for arg in (
+      operations = [
           'clear_labels', 'update_labels', 'remove_labels',
-          'recompute_message_storage_policy')):
+          'recompute_message_storage_policy',
+          'message_storage_policy_allowed_regions'
+      ]
+      if not any(args.IsSpecified(arg) for arg in operations):
         raise
       log.status.Print('No update to perform.')
     else:
