@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*- #
-# Copyright 2018 Google Inc. All Rights Reserved.
+# Copyright 2018 Google LLC. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Deploy an app, function or container to Cloud Run."""
+"""Deploy a container to Cloud Run."""
 
 from __future__ import absolute_import
 from __future__ import division
@@ -20,22 +20,22 @@ from __future__ import print_function
 from __future__ import unicode_literals
 
 from googlecloudsdk.calliope import base
+from googlecloudsdk.command_lib.run import config_changes as config_changes_mod
 from googlecloudsdk.command_lib.run import connection_context
-from googlecloudsdk.command_lib.run import exceptions
 from googlecloudsdk.command_lib.run import flags
 from googlecloudsdk.command_lib.run import pretty_print
 from googlecloudsdk.command_lib.run import resource_args
 from googlecloudsdk.command_lib.run import serverless_operations
 from googlecloudsdk.command_lib.run import stages
+from googlecloudsdk.command_lib.util.args import labels_util
 from googlecloudsdk.command_lib.util.concepts import concept_parsers
 from googlecloudsdk.command_lib.util.concepts import presentation_specs
-from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.console import progress_tracker
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class Deploy(base.Command):
-  """Deploy an app, function or container to Cloud Run."""
+  """Deploy a container to Cloud Run."""
 
   detailed_help = {
       'DESCRIPTION': """\
@@ -67,105 +67,62 @@ class Deploy(base.Command):
         prefixes=False)
     flags.AddSourceRefFlags(parser)
     flags.AddRegionArg(parser)
+    flags.AddPlatformArg(parser)
+    flags.AddKubeconfigFlags(parser)
     flags.AddFunctionArg(parser)
     flags.AddMutexEnvVarsFlags(parser)
+    flags.AddCpuFlag(parser)
     flags.AddMemoryFlag(parser)
     flags.AddConcurrencyFlag(parser)
     flags.AddTimeoutFlag(parser)
     flags.AddAsyncFlag(parser)
     flags.AddEndpointVisibilityEnum(parser)
+    flags.AddCloudSQLFlags(parser)
     flags.AddAllowUnauthenticatedFlag(parser)
+    flags.AddServiceAccountFlag(parser)
     concept_parsers.ConceptParser([
         resource_args.CLUSTER_PRESENTATION,
         service_presentation]).AddToParser(parser)
 
   def Run(self, args):
-    """Deploy an app, function or container to Cloud Run."""
-    source_ref = flags.GetSourceRef(args.source, args.image)
-    config_changes = flags.GetConfigurationChanges(args)
+    """Deploy a container to Cloud Run."""
+    image = args.image
 
     conn_context = connection_context.GetConnectionContext(args)
-
-    if (conn_context.supports_one_platform
-        and getattr(args, 'connectivity', None)):
-      raise exceptions.ConfigurationError(
-          'The `--connectivity=[internal|external]` flag '
-          'is only supported with Cloud Run on GKE.')
-
-    if (not conn_context.supports_one_platform
-        and getattr(args, 'allow_unauthenticated', None)):
-      raise exceptions.ConfigurationError(
-          'The `--allow-unauthenticated` flag '
-          'is not supported with Cloud Run on GKE.')
+    config_changes = flags.GetConfigurationChanges(args)
 
     service_ref = flags.GetService(args)
-    function_entrypoint = flags.GetFunction(args.function)
-    msg = ('Deploying {dep_type} to {operator} '
-           'service [{{bold}}{service}{{reset}}]'
-           ' in {ns_label} [{{bold}}{ns}{{reset}}]')
 
-    msg += conn_context.location_label
+    with serverless_operations.Connect(conn_context) as operations:
+      image_change = config_changes_mod.ImageChange(image)
+      changes = [image_change]
+      if config_changes:
+        changes.extend(config_changes)
+      endpoint_visibility = flags.GetEndpointVisibility(args)
+      exists = operations.GetService(service_ref)
+      allow_unauth = None
+      if conn_context.supports_one_platform:
+        allow_unauth = flags.GetAllowUnauthenticated(args,
+                                                     operations,
+                                                     service_ref,
+                                                     not exists)
+        # Don't try to remove a policy binding from a service that doesn't exist
+        if not exists and not allow_unauth:
+          allow_unauth = None
+
+      msg = ('Deploying {dep_type} to {operator} '
+             'service [{{bold}}{service}{{reset}}]'
+             ' in {ns_label} [{{bold}}{ns}{{reset}}]')
+      msg += conn_context.location_label
 
-    if function_entrypoint:
-      dep_type = 'function [{{bold}}{}{{reset}}]'.format(function_entrypoint)
-      pretty_print.Info(msg.format(
-          operator=conn_context.operator,
-          ns_label=conn_context.ns_label,
-          dep_type=dep_type,
-          function=function_entrypoint,
-          service=service_ref.servicesId,
-          ns=service_ref.namespacesId))
-    elif source_ref.source_type is source_ref.SourceType.IMAGE:
       pretty_print.Info(msg.format(
           operator=conn_context.operator,
           ns_label=conn_context.ns_label,
           dep_type='container',
           service=service_ref.servicesId,
           ns=service_ref.namespacesId))
-    else:
-      pretty_print.Info(msg.format(
-          operator=conn_context.operator,
-          ns_label=conn_context.ns_label,
-          dep_type='app',
-          service=service_ref.servicesId,
-          ns=service_ref.namespacesId))
-
-    with serverless_operations.Connect(conn_context) as operations:
-      if not (source_ref.source_type is source_ref.SourceType.IMAGE
-              or operations.IsSourceBranch()):
-        raise exceptions.SourceNotSupportedError()
-      new_deployable = operations.Detect(service_ref.Parent(),
-                                         source_ref, function_entrypoint)
-      operations.Upload(new_deployable)
-      changes = [new_deployable]
-      if config_changes:
-        changes.extend(config_changes)
-      if args.connectivity == 'internal':
-        private_endpoint = True
-      elif args.connectivity == 'external':
-        private_endpoint = False
-      else:
-        private_endpoint = None
-      deployment_stages = stages.ServiceStages()
-      exists = operations.GetService(service_ref)
-
-      if (not exists and not args.allow_unauthenticated and
-          conn_context.supports_one_platform):
-
-        if operations.CanAddIamPolicyBinding(service_ref):
-          allow_unauth = console_io.PromptContinue(
-              prompt_string=(
-                  'Allow unauthenticated invocations '
-                  'to new service [{}]?'.format(
-                      service_ref.servicesId)),
-              default=False)
-        else:
-          allow_unauth = False
-          pretty_print.Info(
-              'This new service will require authentication to be invoked.')
-      else:
-        allow_unauth = False
 
+      deployment_stages = stages.ServiceStages(allow_unauth is not None)
       header = 'Deploying...' if exists else 'Deploying new service...'
       with progress_tracker.StagedProgressTracker(
           header,
@@ -177,8 +134,8 @@ class Deploy(base.Command):
             changes,
             tracker,
             asyn=args.async,
-            private_endpoint=private_endpoint,
-            allow_unauthenticated=allow_unauth or args.allow_unauthenticated)
+            private_endpoint=endpoint_visibility,
+            allow_unauthenticated=allow_unauth)
       if args.async:
         pretty_print.Success(
             'Service [{{bold}}{serv}{{reset}}] is deploying '
@@ -204,6 +161,6 @@ class AlphaDeploy(Deploy):
   @staticmethod
   def Args(parser):
     Deploy.Args(parser)
-    flags.AddCloudSQLFlags(parser)
+    labels_util.AddUpdateLabelsFlags(parser)
 
 AlphaDeploy.__doc__ = Deploy.__doc__
